@@ -21,13 +21,11 @@
  * Functions to maintain address cache.
  */
 
-#include <glib.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
 
 #include "mgutils.h"
-#include "addritem.h"
 #include "addrcache.h"
 
 #define ID_TIME_OFFSET             998000000
@@ -63,6 +61,7 @@ AddressCache *addrcache_create() {
 	cache->dataRead = FALSE;
 	cache->modified = FALSE;
 	cache->dirtyFlag = FALSE;
+	cache->accessFlag = FALSE;
 	cache->name = NULL;
 	cache->modifyTime = 0;
 
@@ -103,6 +102,30 @@ void addrcache_set_dirty( AddressCache *cache, const gboolean value ) {
 	g_return_if_fail( cache != NULL );
 	cache->dirtyFlag = value;
 }
+gboolean addrcache_get_modified( AddressCache *cache ) {
+	g_return_val_if_fail( cache != NULL, FALSE );
+	return cache->modified;
+}
+void addrcache_set_modified( AddressCache *cache, const gboolean value ) {
+	g_return_if_fail( cache != NULL );
+	cache->modified = value;
+}
+gboolean addrcache_get_read_flag( AddressCache *cache ) {
+	g_return_val_if_fail( cache != NULL, FALSE );
+	return cache->dataRead;
+}
+void addrcache_set_read_flag( AddressCache *cache, const gboolean value ) {
+	g_return_if_fail( cache != NULL );
+	cache->dataRead = value;
+}
+gboolean addrcache_get_accessed( AddressCache *cache ) {
+	g_return_val_if_fail( cache != NULL, FALSE );
+	return cache->accessFlag;
+}
+void addrcache_set_accessed( AddressCache *cache, const gboolean value ) {
+	g_return_if_fail( cache != NULL );
+	cache->accessFlag = value;
+}
 gchar *addrcache_get_name( AddressCache *cache ) {
 	g_return_val_if_fail( cache != NULL, NULL );
 	return cache->name;
@@ -128,6 +151,7 @@ void addrcache_next_id( AddressCache *cache ) {
 void addrcache_refresh( AddressCache *cache ) {
 	cache->dataRead = FALSE;
 	cache->modified = TRUE;
+	cache->accessFlag = FALSE;
 	cache->modifyTime = 0;
 }
 
@@ -136,10 +160,13 @@ void addrcache_refresh( AddressCache *cache ) {
 */
 static gint addrcache_free_item_vis( gpointer key, gpointer value, gpointer data ) {
 	AddrItemObject *obj = ( AddrItemObject * ) value;
+
 	if( ADDRITEM_TYPE(obj) == ITEMTYPE_PERSON ) {
-		/* Free person and their email */
 		addritem_free_item_person( ( ItemPerson * ) obj );
 	}
+	else if( ADDRITEM_TYPE(obj) == ITEMTYPE_EMAIL ) {
+		addritem_free_item_email( ( ItemEMail * ) obj );
+	}
 	else if( ADDRITEM_TYPE(obj) == ITEMTYPE_GROUP ) {
 		addritem_free_item_group( ( ItemGroup * ) obj );
 	}
@@ -148,7 +175,7 @@ static gint addrcache_free_item_vis( gpointer key, gpointer value, gpointer data
 	}
 	key = NULL;
 	value = NULL;
-	return 0;
+	return TRUE;
 }
 
 /*
@@ -159,17 +186,21 @@ static void addrcache_free_item_hash( GHashTable *table ) {
 	g_hash_table_freeze( table );
 	g_hash_table_foreach_remove( table, addrcache_free_item_vis, NULL );
 	g_hash_table_thaw( table );
-	g_hash_table_destroy( table );
 }
 
 /*
 * Free up folders and groups.
 */
 static void addrcache_free_all_folders( ItemFolder *parent ) {
-	GList *node = parent->listFolder;
+	GList *node;
+
+	if( parent == NULL ) return;
+
+	node = parent->listFolder;
 	while( node ) {
 		ItemFolder *folder = node->data;
 		addrcache_free_all_folders( folder );
+		node->data = NULL;
 		node = g_list_next( node );
 	}
 	g_list_free( parent->listPerson );
@@ -186,14 +217,16 @@ static void addrcache_free_all_folders( ItemFolder *parent ) {
 void addrcache_clear( AddressCache *cache ) {
 	g_return_if_fail( cache != NULL );
 
+	/* printf( "...addrcache_clear :%s:\n", cache->name ); */
 	/* Free up folders and hash table */
 	addrcache_free_all_folders( cache->rootFolder );
 	addrcache_free_item_hash( cache->itemHash );
+	g_hash_table_destroy( cache->itemHash );
 	cache->itemHash = NULL;
 	ADDRITEM_PARENT(cache->rootFolder) = NULL;
 	addritem_free_item_folder( cache->rootFolder );
 	cache->rootFolder = NULL;
-	g_list_free( cache->tempList );
+	if( cache->tempList ) g_list_free( cache->tempList );
 	cache->tempList = NULL;
 
 	/* Reset to initial state */
@@ -203,7 +236,6 @@ void addrcache_clear( AddressCache *cache ) {
 	ADDRITEM_PARENT(cache->rootFolder) = NULL;
 
 	addrcache_refresh( cache );
-
 }
 
 /*
@@ -215,6 +247,7 @@ void addrcache_free( AddressCache *cache ) {
 	cache->dirtyFlag = FALSE;
 	addrcache_free_all_folders( cache->rootFolder );
 	addrcache_free_item_hash( cache->itemHash );
+	g_hash_table_destroy( cache->itemHash );
 	cache->itemHash = NULL;
 	ADDRITEM_PARENT(cache->rootFolder) = NULL;
 	addritem_free_item_folder( cache->rootFolder );
@@ -924,11 +957,15 @@ void addrcache_person_move_email(
 		ItemEMail *found;
 		found = addritem_person_remove_email( person, email );
 		if( found ) {
+			/*
 			if( person->listEMail ) {
 				person->listEMail = g_list_remove( person->listEMail, found );
 				addritem_person_add_email( target, found );
 				cache->dirtyFlag = TRUE;
 			}
+			*/
+			addritem_person_add_email( target, found );
+			cache->dirtyFlag = TRUE;
 		}
 	}
 }
@@ -1289,17 +1326,19 @@ ItemFolder *addrcache_remove_folder_delete( AddressCache *cache, ItemFolder *fol
 	return NULL;
 }
 
-/*
-* Add person and address data to cache.
-* Enter: cache     Cache.
-*        folder    Folder where to add person, or NULL for root folder.
-*        name      Common name.
-*        address   EMail address.
-*        remarks   Remarks.
-* Return: Person added. Do not *NOT* to use the addrbook_free_xxx() functions...
-* this will destroy the address book data.
-*/
-ItemPerson *addrcache_add_contact( AddressCache *cache, ItemFolder *folder, const gchar *name,
+/**
+ * Add person and address data to cache.
+ * \param cache     Cache.
+ * \param folder    Folder where to add person, or NULL for root folder.
+ * \param name      Common name.
+ * \param address   EMail address.
+ * \param remarks   Remarks.
+ * \return Person added. Do not *NOT* to use the 
+ *         <code>addrbook_free_xxx()</code> functions...; this will destroy
+ *         the address book data.
+ */
+ItemPerson *addrcache_add_contact(
+		AddressCache *cache, ItemFolder *folder, const gchar *name,
 		const gchar *address, const gchar *remarks )
 {
 	ItemPerson *person = NULL;
@@ -1327,6 +1366,36 @@ ItemPerson *addrcache_add_contact( AddressCache *cache, ItemFolder *folder, cons
 	return person;
 }
 
+/**
+ * Create a new folder and add to address cache.
+ * \param  cache  Address cache.
+ * \param  folder Parent folder where to add folder, or <i>NULL</i> for
+ *                root folder.
+ * \return Folder that was created. This should <b>*NOT*</b> be
+ *         <code>g_free()</code> when done.
+ */
+ItemFolder *addrcache_add_new_folder( AddressCache *cache, ItemFolder *parent )
+{
+	ItemFolder *folder;
+	ItemFolder *p = parent;
+
+	g_return_val_if_fail( cache != NULL, NULL );
+
+	if( !p ) p = cache->rootFolder;
+	folder = addritem_create_item_folder();
+	addrcache_id_folder( cache, folder );
+	if( addrcache_hash_add_folder( cache, folder ) ) {
+		p->listFolder = g_list_append( p->listFolder, folder );
+		ADDRITEM_PARENT(folder) = ADDRITEM_OBJECT(p);
+		addrcache_set_dirty( cache, TRUE );
+	}
+	else {
+		addritem_free_item_folder( folder );
+		folder = NULL;
+	}
+	return folder;
+}
+
 /*
 * End of Source.
 */
