@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,8 +70,6 @@ static struct _GroupEdit_dlg {
 	GtkCList *clist_avail;
 
 	GHashTable *hashEMail;
-	gint rowIndGroup;
-	gint rowIndAvail;
 
 } groupeditdlg;
 
@@ -88,8 +86,10 @@ static void edit_group_status_show( gchar *msg ) {
 }
 
 static void edit_group_ok(GtkWidget *widget, gboolean *cancelled) {
-	gchar *sName = g_strdup( gtk_editable_get_chars( GTK_EDITABLE(groupeditdlg.entry_name), 0, -1 ) );
+	gchar *sName;
 	gboolean errFlag = TRUE;
+
+	sName = gtk_editable_get_chars( GTK_EDITABLE(groupeditdlg.entry_name), 0, -1 );
 	if( sName ) {
 		g_strstrip( sName );
 		if( *sName != '\0' ) {
@@ -163,13 +163,6 @@ static void edit_group_load_clist( GtkCList *clist, GList *listEMail ) {
 	}
 }
 
-static void edit_group_group_selected( GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer data ) {
-	groupeditdlg.rowIndGroup = row;
-}
-
-static void edit_group_avail_selected( GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer data ) {
-	groupeditdlg.rowIndAvail = row;
-}
 
 static gint edit_group_move_email( GtkCList *clist_from, GtkCList *clist_to, gint row ) {
 	ItemEMail *email = gtk_clist_get_row_data( clist_from, row );
@@ -183,13 +176,25 @@ static gint edit_group_move_email( GtkCList *clist_from, GtkCList *clist_to, gin
 }
 
 static void edit_group_to_group( GtkWidget *widget, gpointer data ) {
-	groupeditdlg.rowIndGroup = edit_group_move_email( groupeditdlg.clist_avail,
-					groupeditdlg.clist_group, groupeditdlg.rowIndAvail );
+	GList *selected = GTK_CLIST(groupeditdlg.clist_avail)->selection;
+	/* Clear the selected rows on destination clist */
+	gtk_clist_unselect_all(groupeditdlg.clist_group);
+	while (selected) {
+		edit_group_move_email( groupeditdlg.clist_avail,
+					groupeditdlg.clist_group, GPOINTER_TO_UINT(selected->data) );
+		/* cannot use g_list_next as the selection list will have changed */
+		selected = GTK_CLIST(groupeditdlg.clist_avail)->selection;
+	}
 }
 
 static void edit_group_to_avail( GtkWidget *widget, gpointer data ) {
-	groupeditdlg.rowIndAvail = edit_group_move_email( groupeditdlg.clist_group,
-					groupeditdlg.clist_avail, groupeditdlg.rowIndGroup );
+	GList *selected = GTK_CLIST(groupeditdlg.clist_group)->selection;
+	gtk_clist_unselect_all(groupeditdlg.clist_avail);
+	while (selected) {
+		edit_group_move_email( groupeditdlg.clist_group,
+					groupeditdlg.clist_avail, GPOINTER_TO_UINT(selected->data) );
+		selected = GTK_CLIST(groupeditdlg.clist_group)->selection;
+	}
 }
 
 static void edit_group_list_group_button( GtkCList *clist, GdkEventButton *event, gpointer data ) {
@@ -316,7 +321,7 @@ static void addressbook_edit_group_create( gboolean *cancelled ) {
 
 	clist_group = gtk_clist_new_with_titles( GROUP_N_COLS, titles );
 	gtk_container_add( GTK_CONTAINER(clist_swin), clist_group );
-	gtk_clist_set_selection_mode( GTK_CLIST(clist_group), GTK_SELECTION_BROWSE );
+	gtk_clist_set_selection_mode( GTK_CLIST(clist_group), GTK_SELECTION_EXTENDED );
 	gtk_clist_set_column_width( GTK_CLIST(clist_group), GROUP_COL_NAME, GROUP_COL_WIDTH_NAME );
 	gtk_clist_set_column_width( GTK_CLIST(clist_group), GROUP_COL_EMAIL, GROUP_COL_WIDTH_EMAIL );
 	gtk_clist_set_compare_func( GTK_CLIST(clist_group), edit_group_list_compare_func );
@@ -345,7 +350,7 @@ static void addressbook_edit_group_create( gboolean *cancelled ) {
 
 	clist_avail = gtk_clist_new_with_titles( GROUP_N_COLS, titles );
 	gtk_container_add( GTK_CONTAINER(clist_swin), clist_avail );
-	gtk_clist_set_selection_mode( GTK_CLIST(clist_avail), GTK_SELECTION_BROWSE );
+	gtk_clist_set_selection_mode( GTK_CLIST(clist_avail), GTK_SELECTION_EXTENDED );
 	gtk_clist_set_column_width( GTK_CLIST(clist_avail), GROUP_COL_NAME, GROUP_COL_WIDTH_NAME );
 	gtk_clist_set_column_width( GTK_CLIST(clist_avail), GROUP_COL_EMAIL, GROUP_COL_WIDTH_EMAIL );
 	gtk_clist_set_compare_func( GTK_CLIST(clist_avail), edit_group_list_compare_func );
@@ -374,10 +379,6 @@ static void addressbook_edit_group_create( gboolean *cancelled ) {
 	gtk_widget_show_all(vbox);
 
 	/* Event handlers */
-	gtk_signal_connect( GTK_OBJECT(clist_group), "select_row",
-			GTK_SIGNAL_FUNC( edit_group_group_selected), NULL );
-	gtk_signal_connect( GTK_OBJECT(clist_avail), "select_row",
-			GTK_SIGNAL_FUNC( edit_group_avail_selected), NULL );
 	gtk_signal_connect( GTK_OBJECT(buttonGroup), "clicked",
 			GTK_SIGNAL_FUNC( edit_group_to_group ), NULL );
 	gtk_signal_connect( GTK_OBJECT(buttonAvail), "clicked",
@@ -428,6 +429,7 @@ static GList *edit_group_build_email_list() {
 ItemGroup *addressbook_edit_group( AddressBookFile *abf, ItemFolder *parent, ItemGroup *group ) {
 	static gboolean cancelled;
 	GList *listEMail = NULL;
+	gchar *name;
 
 	if (!groupeditdlg.window)
 		addressbook_edit_group_create(&cancelled);
@@ -437,8 +439,6 @@ ItemGroup *addressbook_edit_group( AddressBookFile *abf, ItemFolder *parent, Ite
 	manage_window_set_transient(GTK_WINDOW(groupeditdlg.window));
 
 	/* Clear all fields */
-	groupeditdlg.rowIndGroup = -1;
-	groupeditdlg.rowIndAvail = -1;
 	edit_group_status_show( "" );
 	gtk_clist_clear( GTK_CLIST(groupeditdlg.clist_group) );
 	gtk_clist_clear( GTK_CLIST(groupeditdlg.clist_avail) );
@@ -479,7 +479,9 @@ ItemGroup *addressbook_edit_group( AddressBookFile *abf, ItemFolder *parent, Ite
 		/* Create new person and email list */
 		group = addrbook_add_group_list( abf, parent, listEMail );
 	}
-	addritem_group_set_name( group, gtk_editable_get_chars( GTK_EDITABLE(groupeditdlg.entry_name), 0, -1 ) );
+	name = gtk_editable_get_chars( GTK_EDITABLE(groupeditdlg.entry_name), 0, -1 );
+	addritem_group_set_name( group, name );
+	g_free( name );
 
 	listEMail = NULL;
 	return group;
@@ -512,7 +514,7 @@ ItemFolder *addressbook_edit_folder( AddressBookFile *abf, ItemFolder *parent, I
 		return NULL;
 	}
 	if( folder ) {
-		if( strcasecmp( name, ADDRITEM_NAME(folder) ) == 0 ) {
+		if( strcmp( name, ADDRITEM_NAME(folder) ) == 0 ) {
 			g_free( name );
 			return NULL;
 		}
