@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,24 +40,60 @@
 #include "procheader.h"
 #include "folder.h"
 #include "session.h"
-#include "statusbar.h"
 #include "codeconv.h"
 #include "utils.h"
 #include "prefs_common.h"
 #include "prefs_account.h"
 #include "inputdialog.h"
+#include "log.h"
+#include "progressindicator.h"
+#include "remotefolder.h"
 #include "alertpanel.h"
+#if USE_OPENSSL
+#  include "ssl.h"
+#endif
 
 #define NNTP_PORT	119
+#if USE_OPENSSL
+#define NNTPS_PORT	563
+#endif
+
+typedef struct _NewsFolder	NewsFolder;
+
+#define NEWS_FOLDER(obj)	((NewsFolder *)obj)
+
+struct _NewsFolder
+{
+	RemoteFolder rfolder;
+
+	gboolean use_auth;
+};
 
 static void news_folder_init		 (Folder	*folder,
 					  const gchar	*name,
 					  const gchar	*path);
 
+static Folder	*news_folder_new	(const gchar	*name,
+					 const gchar	*folder);
+static void	 news_folder_destroy	(Folder		*folder);
+
+static gchar *news_fetch_msg		(Folder		*folder,
+					 FolderItem	*item,
+					 gint		 num);
+
+
+#if USE_OPENSSL
+static Session *news_session_new	 (const gchar	*server,
+					  gushort	 port,
+					  const gchar	*userid,
+					  const gchar	*passwd,
+					  SSLType	 ssl_type);
+#else
 static Session *news_session_new	 (const gchar	*server,
 					  gushort	 port,
 					  const gchar	*userid,
 					  const gchar	*passwd);
+#endif
 
 static gint news_get_article_cmd	 (NNTPSession	*session,
 					  const gchar	*cmd,
@@ -66,44 +102,81 @@ static gint news_get_article_cmd	 (NNTPSession	*session,
 static gint news_get_article		 (NNTPSession	*session,
 					  gint		 num,
 					  gchar		*filename);
-static gint news_get_header		 (NNTPSession	*session,
-					  gint		 num,
-					  gchar		*filename);
 
 static gint news_select_group		 (NNTPSession	*session,
 					  const gchar	*group,
 					  gint		*num,
 					  gint		*first,
 					  gint		*last);
-static GSList *news_get_uncached_articles(NNTPSession	*session,
-					  FolderItem	*item,
-					  gint		 cache_last,
-					  gint		*rfirst,
-					  gint		*rlast);
 static MsgInfo *news_parse_xover	 (const gchar	*xover_str);
 static gchar *news_parse_xhdr		 (const gchar	*xhdr_str,
 					  MsgInfo	*msginfo);
-static GSList *news_delete_old_articles	 (GSList	*alist,
-					  FolderItem	*item,
-					  gint		 first);
-static void news_delete_all_articles	 (FolderItem	*item);
+gint news_get_num_list		 	 (Folder 	*folder, 
+					  FolderItem 	*item,
+					  GSList       **list,
+					  gboolean	*old_uids_valid);
+MsgInfo *news_get_msginfo		 (Folder 	*folder, 
+					  FolderItem 	*item,
+					  gint 		 num);
+GSList *news_get_msginfos		 (Folder 	*folder,
+					  FolderItem 	*item,
+					  GSList 	*msgnum_list);
+gboolean news_scan_required		 (Folder 	*folder,
+					  FolderItem 	*item);
 
-static gint news_remove_msg		 (Folder	*folder, 
-					  FolderItem	*item, 
-					  gint		 num);
+gint news_post_stream			 (Folder 	*folder, 
+					  FILE 		*fp);
+static gchar *news_folder_get_path	 (Folder	*folder);
+gchar *news_item_get_path		 (Folder	*folder,
+					  FolderItem	*item);
 
-Folder *news_folder_new(const gchar *name, const gchar *path)
+static FolderClass news_class;
+
+FolderClass *news_get_class(void)
+{
+	if (news_class.idstr == NULL) {
+		news_class.type = F_NEWS;
+		news_class.idstr = "news";
+		news_class.uistr = "News";
+
+		/* Folder functions */
+		news_class.new_folder = news_folder_new;
+		news_class.destroy_folder = news_folder_destroy;
+
+		/* FolderItem functions */
+		news_class.item_get_path = news_item_get_path;
+		news_class.get_num_list = news_get_num_list;
+		news_class.scan_required = news_scan_required;
+
+		/* Message functions */
+		news_class.get_msginfo = news_get_msginfo;
+		news_class.get_msginfos = news_get_msginfos;
+		news_class.fetch_msg = news_fetch_msg;
+	};
+
+	return &news_class;
+}
+
+static Folder *news_folder_new(const gchar *name, const gchar *path)
 {
 	Folder *folder;
 
 	folder = (Folder *)g_new0(NewsFolder, 1);
+	folder->klass = &news_class;
 	news_folder_init(folder, name, path);
 
 	return folder;
 }
 
-void news_folder_destroy(NewsFolder *folder)
+static void news_folder_destroy(Folder *folder)
 {
+	gchar *dir;
+
+	dir = news_folder_get_path(folder);
+	if (is_dir_exist(dir))
+		remove_dir_recursive(dir);
+	g_free(dir);
+
 	folder_remote_folder_destroy(REMOTE_FOLDER(folder));
 }
 
@@ -111,54 +184,31 @@ static void news_folder_init(Folder *folder, const gchar *name,
 			     const gchar *path)
 {
 	folder_remote_folder_init(folder, name, path);
-
-	folder->type = F_NEWS;
-
-	folder->get_msg_list = news_get_article_list;
-	folder->fetch_msg    = news_fetch_msg;
-	folder->scan         = news_scan_group;
-	folder->remove_msg   = news_remove_msg;
 }
 
+#if USE_OPENSSL
+static Session *news_session_new(const gchar *server, gushort port,
+				 const gchar *userid, const gchar *passwd,
+				 SSLType ssl_type)
+#else
 static Session *news_session_new(const gchar *server, gushort port,
 				 const gchar *userid, const gchar *passwd)
+#endif
 {
 	gchar buf[NNTPBUFSIZE];
-	NNTPSession *session;
-	NNTPSockInfo *nntp_sock;
+	Session *session;
 
 	g_return_val_if_fail(server != NULL, NULL);
 
 	log_message(_("creating NNTP connection to %s:%d ...\n"), server, port);
 
-	if (userid && passwd)
-		nntp_sock = nntp_open_auth(server, port, buf, userid, passwd);
-	else
-		nntp_sock = nntp_open(server, port, buf);
-	if (nntp_sock == NULL)
-		return NULL;
+#if USE_OPENSSL
+	session = nntp_session_new(server, port, buf, userid, passwd, ssl_type);
+#else
+	session = nntp_session_new(server, port, buf, userid, passwd);
+#endif
 
-	session = g_new(NNTPSession, 1);
-	SESSION(session)->type             = SESSION_NEWS;
-	SESSION(session)->server           = g_strdup(server);
-	session->nntp_sock                 = nntp_sock;
-	SESSION(session)->sock             = nntp_sock->sock;
-	SESSION(session)->connected        = TRUE;
-	SESSION(session)->phase            = SESSION_READY;
-	SESSION(session)->last_access_time = time(NULL);
-	SESSION(session)->data             = NULL;
-	session->group = NULL;
-
-	return SESSION(session);
-}
-
-void news_session_destroy(NNTPSession *session)
-{
-	nntp_close(session->nntp_sock);
-	session->nntp_sock = NULL;
-	SESSION(session)->sock = NULL;
-
-	g_free(session->group);
+	return session;
 }
 
 static Session *news_session_new_for_folder(Folder *folder)
@@ -167,6 +217,8 @@ static Session *news_session_new_for_folder(Folder *folder)
 	PrefsAccount *ac;
 	const gchar *userid = NULL;
 	gchar *passwd = NULL;
+	gushort port;
+	gchar buf[NNTPBUFSIZE];
 
 	g_return_val_if_fail(folder != NULL, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
@@ -181,35 +233,45 @@ static Session *news_session_new_for_folder(Folder *folder)
 							     userid);
 	}
 
-	session = news_session_new(ac->nntp_server,
-				   ac->set_nntpport ? ac->nntpport : NNTP_PORT,
-				   userid, passwd);
+#if USE_OPENSSL
+	port = ac->set_nntpport ? ac->nntpport
+		: ac->ssl_nntp ? NNTPS_PORT : NNTP_PORT;
+	session = news_session_new(ac->nntp_server, port, userid, passwd,
+				   ac->ssl_nntp);
+#else
+	port = ac->set_nntpport ? ac->nntpport : NNTP_PORT;
+	session = news_session_new(ac->nntp_server, port, userid, passwd);
+#endif
+	if ((session != NULL) && ac->use_nntp_auth && ac->use_nntp_auth_onconnect)
+		nntp_forceauth(NNTP_SESSION(session), buf, userid, passwd);
+
 	g_free(passwd);
 
 	return session;
 }
 
-NNTPSession *news_session_get(Folder *folder)
+static NNTPSession *news_session_get(Folder *folder)
 {
 	RemoteFolder *rfolder = REMOTE_FOLDER(folder);
 
 	g_return_val_if_fail(folder != NULL, NULL);
-	g_return_val_if_fail(folder->type == F_NEWS, NULL);
+	g_return_val_if_fail(FOLDER_CLASS(folder) == &news_class, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
 
+	if (prefs_common.work_offline)
+		return NULL;
+
 	if (!rfolder->session) {
 		rfolder->session = news_session_new_for_folder(folder);
-		statusbar_pop_all();
 		return NNTP_SESSION(rfolder->session);
 	}
 
 	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT) {
 		rfolder->session->last_access_time = time(NULL);
-		statusbar_pop_all();
 		return NNTP_SESSION(rfolder->session);
 	}
 
-	if (nntp_mode(NNTP_SESSION(rfolder->session)->nntp_sock, FALSE)
+	if (nntp_mode(NNTP_SESSION(rfolder->session), FALSE)
 	    != NN_SUCCESS) {
 		log_warning(_("NNTP connection to %s:%d has been"
 			      " disconnected. Reconnecting...\n"),
@@ -222,56 +284,10 @@ NNTPSession *news_session_get(Folder *folder)
 
 	if (rfolder->session)
 		rfolder->session->last_access_time = time(NULL);
-	statusbar_pop_all();
 	return NNTP_SESSION(rfolder->session);
 }
 
-GSList *news_get_article_list(Folder *folder, FolderItem *item,
-			      gboolean use_cache)
-{
-	GSList *alist;
-	NNTPSession *session;
-
-	g_return_val_if_fail(folder != NULL, NULL);
-	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(folder->type == F_NEWS, NULL);
-
-	session = news_session_get(folder);
-
-	if (!session) {
-		alist = procmsg_read_cache(item, FALSE);
-		item->last_num = procmsg_get_last_num_in_cache(alist);
-	} else if (use_cache) {
-		GSList *newlist;
-		gint cache_last;
-		gint first, last;
-
-		alist = procmsg_read_cache(item, FALSE);
-
-		cache_last = procmsg_get_last_num_in_cache(alist);
-		newlist = news_get_uncached_articles
-			(session, item, cache_last, &first, &last);
-		alist = news_delete_old_articles(alist, item, first);
-
-		alist = g_slist_concat(alist, newlist);
-		item->last_num = last;
-	} else {
-		gint last;
-
-		alist = news_get_uncached_articles
-			(session, item, 0, NULL, &last);
-		news_delete_all_articles(item);
-		item->last_num = last;
-	}
-
-	procmsg_set_flags(alist, item);
-
-	statusbar_pop_all();
-
-	return alist;
-}
-
-gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
+static gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 {
 	gchar *path, *filename;
 	NNTPSession *session;
@@ -287,7 +303,7 @@ gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	g_free(path);
 
 	if (is_file_exist(filename)) {
-		debug_print(_("article %d has been already cached.\n"), num);
+		debug_print("article %d has been already cached.\n", num);
 		return filename;
 	}
 
@@ -298,19 +314,24 @@ gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	}
 
 	ok = news_select_group(session, item->path, NULL, NULL, NULL);
-	statusbar_pop_all();
 	if (ok != NN_SUCCESS) {
-		g_warning(_("can't select group %s\n"), item->path);
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(folder)->session = NULL;
+		}
 		g_free(filename);
 		return NULL;
 	}
 
-	debug_print(_("getting article %d...\n"), num);
+	debug_print("getting article %d...\n", num);
 	ok = news_get_article(NNTP_SESSION(REMOTE_FOLDER(folder)->session),
 			      num, filename);
-	statusbar_pop_all();
-	if (ok < 0) {
-		g_warning(_("can't read article %d\n"), num);
+	if (ok != NN_SUCCESS) {
+		g_warning("can't read article %d\n", num);
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(folder)->session = NULL;
+		}
 		g_free(filename);
 		return NULL;
 	}
@@ -318,55 +339,6 @@ gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	return filename;
 }
 
-void news_scan_group(Folder *folder, FolderItem *item)
-{
-	NNTPSession *session;
-	gint num = 0, first = 0, last = 0;
-	gint new = 0, unread = 0, total = 0;
-	gint min = 0, max = 0;
-	gchar *path;
-	gint ok;
-
-	g_return_if_fail(folder != NULL);
-	g_return_if_fail(item != NULL);
-
-	session = news_session_get(folder);
-	if (!session) return;
-
-	ok = news_select_group(session, item->path, &num, &first, &last);
-	if (ok != NN_SUCCESS) {
-		log_warning(_("can't set group: %s\n"), item->path);
-		return;
-	}
-
-	if (num == 0) {
-		item->new = item->unread = item->total = item->last_num = 0;
-		return;
-	}
-
-	path = folder_item_get_path(item);
-	if (path && is_dir_exist(path)) {
-		procmsg_get_mark_sum(path, &new, &unread, &total, &min, &max,
-				     first);
-	}
-	g_free(path);
-
-	if (first < min) {
-		new = unread = total = num;
-	} else if (max < first) {
-		new = unread = total = num;
-	} else if (last > max) {
-		new += last - max;
-		unread += last - max;
-		if (new > num) new = num;
-		if (unread > num) unread = num;
-	}
-	item->new = new;
-	item->unread = unread;
-	item->total = num;
-	item->last_num = last;
-}
-
 static NewsGroupInfo *news_group_info_new(const gchar *name,
 					  gint first, gint last, gchar type)
 {
@@ -402,7 +374,7 @@ GSList *news_get_group_list(Folder *folder)
 	gchar buf[NNTPBUFSIZE];
 
 	g_return_val_if_fail(folder != NULL, NULL);
-	g_return_val_if_fail(folder->type == F_NEWS, NULL);
+	g_return_val_if_fail(FOLDER_CLASS(folder) == &news_class, NULL);
 
 	path = folder_item_get_path(FOLDER_ITEM(folder->node->data));
 	if (!is_dir_exist(path))
@@ -410,8 +382,9 @@ GSList *news_get_group_list(Folder *folder)
 	filename = g_strconcat(path, G_DIR_SEPARATOR_S, NEWSGROUP_LIST, NULL);
 	g_free(path);
 
-	if ((fp = fopen(filename, "r")) == NULL) {
+	if ((fp = fopen(filename, "rb")) == NULL) {
 		NNTPSession *session;
+		gint ok;
 
 		session = news_session_get(folder);
 		if (!session) {
@@ -419,12 +392,15 @@ GSList *news_get_group_list(Folder *folder)
 			return NULL;
 		}
 
-		if (nntp_list(session->nntp_sock) != NN_SUCCESS) {
+		ok = nntp_list(session);
+		if (ok != NN_SUCCESS) {
+			if (ok == NN_SOCKET) {
+				session_destroy(SESSION(session));
+				REMOTE_FOLDER(folder)->session = NULL;
+			}
 			g_free(filename);
-			statusbar_pop_all();
 			return NULL;
 		}
-		statusbar_pop_all();
 		if (recv_write_to_file(SESSION(session)->sock, filename) < 0) {
 			log_warning(_("can't retrieve newsgroup list\n"));
 			session_destroy(SESSION(session));
@@ -433,7 +409,7 @@ GSList *news_get_group_list(Folder *folder)
 			return NULL;
 		}
 
-		if ((fp = fopen(filename, "r")) == NULL) {
+		if ((fp = fopen(filename, "rb")) == NULL) {
 			FILE_OP_ERROR(filename, "fopen");
 			g_free(filename);
 			return NULL;
@@ -472,8 +448,6 @@ GSList *news_get_group_list(Folder *folder)
 
 	list = g_slist_sort(list, (GCompareFunc)news_group_info_compare);
 
-	statusbar_pop_all();
-
 	return list;
 }
 
@@ -493,7 +467,7 @@ void news_remove_group_list_cache(Folder *folder)
 	gchar *path, *filename;
 
 	g_return_if_fail(folder != NULL);
-	g_return_if_fail(folder->type == F_NEWS);
+	g_return_if_fail(FOLDER_CLASS(folder) == &news_class);
 
 	path = folder_item_get_path(FOLDER_ITEM(folder->node->data));
 	filename = g_strconcat(path, G_DIR_SEPARATOR_S, NEWSGROUP_LIST, NULL);
@@ -508,31 +482,46 @@ void news_remove_group_list_cache(Folder *folder)
 
 gint news_post(Folder *folder, const gchar *file)
 {
-	NNTPSession *session;
 	FILE *fp;
 	gint ok;
 
 	g_return_val_if_fail(folder != NULL, -1);
-	g_return_val_if_fail(folder->type == F_NEWS, -1);
+	g_return_val_if_fail(FOLDER_CLASS(folder) == &news_class, -1);
 	g_return_val_if_fail(file != NULL, -1);
 
-	session = news_session_get(folder);
-	if (!session) return -1;
-
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return -1;
 	}
 
-	ok = nntp_post(session->nntp_sock, fp);
-	if (ok != NN_SUCCESS) {
-		log_warning(_("can't post article.\n"));
-		return -1;
-	}
+	ok = news_post_stream(folder, fp);
 
 	fclose(fp);
 
-	statusbar_pop_all();
+	return ok;
+}
+
+gint news_post_stream(Folder *folder, FILE *fp)
+{
+	NNTPSession *session;
+	gint ok;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(FOLDER_CLASS(folder) == &news_class, -1);
+	g_return_val_if_fail(fp != NULL, -1);
+
+	session = news_session_get(folder);
+	if (!session) return -1;
+
+	ok = nntp_post(session, fp);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't post article.\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(folder)->session = NULL;
+		}
+		return -1;
+	}
 
 	return 0;
 }
@@ -541,42 +530,25 @@ static gint news_get_article_cmd(NNTPSession *session, const gchar *cmd,
 				 gint num, gchar *filename)
 {
 	gchar *msgid;
+	gint ok;
 
-	if (nntp_get_article(session->nntp_sock, cmd, num, &msgid)
-	    != NN_SUCCESS)
-		return -1;
+	ok = nntp_get_article(session, cmd, num, &msgid);
+	if (ok != NN_SUCCESS)
+		return ok;
 
 	debug_print("Message-Id = %s, num = %d\n", msgid, num);
 	g_free(msgid);
 
-	if (recv_write_to_file(session->nntp_sock->sock, filename) < 0) {
+	ok = recv_write_to_file(SESSION(session)->sock, filename);
+	if (ok < 0) {
 		log_warning(_("can't retrieve article %d\n"), num);
-		return -1;
+		if (ok == -2)
+			return NN_SOCKET;
+		else
+			return NN_IOERR;
 	}
 
-	return 0;
-}
-
-static gint news_remove_msg(Folder *folder, FolderItem *item, gint num)
-{
-	MsgInfo * msginfo;
-	gchar * filename;
-	MsgFlags msgflags = { 0, 0 };
-	gint r;
-
-	filename = folder_item_fetch_msg(item, num);
-	if (filename == NULL)
-		return -1;
-
-	msginfo = procheader_parse(filename, msgflags, FALSE, FALSE);
-	if (msginfo == NULL)
-		return -1;
-
-	r = news_cancel_article(folder, msginfo);
-
-	procmsg_msginfo_free(msginfo);
-
-	return r;
+	return NN_SUCCESS;
 }
 
 static gint news_get_article(NNTPSession *session, gint num, gchar *filename)
@@ -584,11 +556,6 @@ static gint news_get_article(NNTPSession *session, gint num, gchar *filename)
 	return news_get_article_cmd(session, "ARTICLE", num, filename);
 }
 
-static gint news_get_header(NNTPSession *session, gint num, gchar *filename)
-{
-	return news_get_article_cmd(session, "HEAD", num, filename);
-}
-
 /**
  * news_select_group:
  * @session: Active NNTP session.
@@ -619,149 +586,15 @@ static gint news_select_group(NNTPSession *session, const gchar *group,
 	g_free(session->group);
 	session->group = NULL;
 
-	ok = nntp_group(session->nntp_sock, group, num, first, last);
+	ok = nntp_group(session, group, num, first, last);
 	if (ok == NN_SUCCESS)
 		session->group = g_strdup(group);
+	else
+		log_warning(_("can't select group: %s\n"), group);
 
 	return ok;
 }
 
-static GSList *news_get_uncached_articles(NNTPSession *session,
-					  FolderItem *item, gint cache_last,
-					  gint *rfirst, gint *rlast)
-{
-	gint ok;
-	gint num = 0, first = 0, last = 0, begin = 0, end = 0;
-	gchar buf[NNTPBUFSIZE];
-	GSList *newlist = NULL;
-	GSList *llast = NULL;
-	MsgInfo *msginfo;
-
-	if (rfirst) *rfirst = 0;
-	if (rlast)  *rlast  = 0;
-
-	g_return_val_if_fail(session != NULL, NULL);
-	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(item->folder != NULL, NULL);
-	g_return_val_if_fail(item->folder->type == F_NEWS, NULL);
-
-	ok = news_select_group(session, item->path, &num, &first, &last);
-	if (ok != NN_SUCCESS) {
-		log_warning(_("can't set group: %s\n"), item->path);
-		return NULL;
-	}
-
-	/* calculate getting overview range */
-	if (first > last) {
-		log_warning(_("invalid article range: %d - %d\n"),
-			    first, last);
-		return NULL;
-	}
-	if (cache_last < first)
-		begin = first;
-	else if (last < cache_last)
-		begin = first;
-	else if (last == cache_last) {
-		debug_print(_("no new articles.\n"));
-		return NULL;
-	} else
-		begin = cache_last + 1;
-	end = last;
-
-	if (rfirst) *rfirst = first;
-	if (rlast)  *rlast  = last;
-
-	if (prefs_common.max_articles > 0 &&
-	    end - begin + 1 > prefs_common.max_articles)
-		begin = end - prefs_common.max_articles + 1;
-
-	log_message(_("getting xover %d - %d in %s...\n"),
-		    begin, end, item->path);
-	if (nntp_xover(session->nntp_sock, begin, end) != NN_SUCCESS) {
-		log_warning(_("can't get xover\n"));
-		return NULL;
-	}
-
-	for (;;) {
-		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
-			log_warning(_("error occurred while getting xover.\n"));
-			return newlist;
-		}
-
-		if (buf[0] == '.' && buf[1] == '\r') break;
-
-		msginfo = news_parse_xover(buf);
-		if (!msginfo) {
-			log_warning(_("invalid xover line: %s\n"), buf);
-			continue;
-		}
-
-		msginfo->folder = item;
-		msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
-		msginfo->flags.tmp_flags = MSG_NEWS;
-		msginfo->newsgroups = g_strdup(item->path);
-
-		if (!newlist)
-			llast = newlist = g_slist_append(newlist, msginfo);
-		else {
-			llast = g_slist_append(llast, msginfo);
-			llast = llast->next;
-		}
-	}
-
-	if (nntp_xhdr(session->nntp_sock, "to", begin, end) != NN_SUCCESS) {
-		log_warning(_("can't get xhdr\n"));
-		return newlist;
-	}
-
-	llast = newlist;
-
-	for (;;) {
-		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
-			log_warning(_("error occurred while getting xhdr.\n"));
-			return newlist;
-		}
-
-		if (buf[0] == '.' && buf[1] == '\r') break;
-		if (!llast) {
-			g_warning("llast == NULL\n");
-			continue;
-		}
-
-		msginfo = (MsgInfo *)llast->data;
-		msginfo->to = news_parse_xhdr(buf, msginfo);
-
-		llast = llast->next;
-	}
-
-	if (nntp_xhdr(session->nntp_sock, "cc", begin, end) != NN_SUCCESS) {
-		log_warning(_("can't get xhdr\n"));
-		return newlist;
-	}
-
-	llast = newlist;
-
-	for (;;) {
-		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
-			log_warning(_("error occurred while getting xhdr.\n"));
-			return newlist;
-		}
-
-		if (buf[0] == '.' && buf[1] == '\r') break;
-		if (!llast) {
-			g_warning("llast == NULL\n");
-			continue;
-		}
-
-		msginfo = (MsgInfo *)llast->data;
-		msginfo->cc = news_parse_xhdr(buf, msginfo);
-
-		llast = llast->next;
-	}
-
-	return newlist;
-}
-
 #define PARSE_ONE_PARAM(p, srcp) \
 { \
 	p = strchr(srcp, '\t'); \
@@ -788,6 +621,13 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
 	PARSE_ONE_PARAM(ref, msgid);
 	PARSE_ONE_PARAM(size, ref);
 	PARSE_ONE_PARAM(line, size);
+	/*
+	 * PARSE_ONE_PARAM(xref, line);
+	 *
+         * if we parse extra headers we should first examine the
+	 * LIST OVERVIEW.FMT response from the server. See
+	 * RFC2980 for details
+	 */
 
 	tmp = strchr(line, '\t');
 	if (!tmp) tmp = strchr(line, '\r');
@@ -799,7 +639,7 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
 	line_int = atoi(line);
 
 	/* set MsgInfo */
-	msginfo = g_new0(MsgInfo, 1);
+	msginfo = procmsg_msginfo_new();
 	msginfo->msgnum = num;
 	msginfo->size = size_int;
 
@@ -827,6 +667,15 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
 			msginfo->inreplyto = g_strdup(p);
 	}
 
+	/*
+	msginfo->xref = g_strdup(xref);
+	p = msginfo->xref+strlen(msginfo->xref) - 1;
+	while (*p == '\r' || *p == '\n') {
+		*p = '\0';
+		p--;
+	}
+	*/
+
 	return msginfo;
 }
 
@@ -854,57 +703,6 @@ static gchar *news_parse_xhdr(const gchar *xhdr_str, MsgInfo *msginfo)
 		return g_strdup(p);
 }
 
-static GSList *news_delete_old_articles(GSList *alist, FolderItem *item,
-					gint first)
-{
-	GSList *cur, *next;
-	MsgInfo *msginfo;
-	gchar *dir;
-
-	g_return_val_if_fail(item != NULL, alist);
-	g_return_val_if_fail(item->folder != NULL, alist);
-	g_return_val_if_fail(item->folder->type == F_NEWS, alist);
-
-	if (first < 2) return alist;
-
-	debug_print(_("Deleting cached articles 1 - %d ... "), first - 1);
-
-	dir = folder_item_get_path(item);
-	remove_numbered_files(dir, 1, first - 1);
-	g_free(dir);
-
-	for (cur = alist; cur != NULL; ) {
-		next = cur->next;
-
-		msginfo = (MsgInfo *)cur->data;
-		if (msginfo && msginfo->msgnum < first) {
-			procmsg_msginfo_free(msginfo);
-			alist = g_slist_remove(alist, msginfo);
-		}
-
-		cur = next;
-	}
-
-	return alist;
-}
-
-static void news_delete_all_articles(FolderItem *item)
-{
-	gchar *dir;
-
-	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_NEWS);
-
-	debug_print(_("\tDeleting all cached articles... "));
-
-	dir = folder_item_get_path(item);
-	remove_all_numbered_files(dir);
-	g_free(dir);
-
-	debug_print(_("done.\n"));
-}
-
 gint news_cancel_article(Folder * folder, MsgInfo * msginfo)
 {
 	gchar * tmp;
@@ -916,13 +714,13 @@ gint news_cancel_article(Folder * folder, MsgInfo * msginfo)
 	if (tmp == NULL)
 		return -1;
 
-	if ((tmpfp = fopen(tmp, "w")) == NULL) {
+	if ((tmpfp = fopen(tmp, "wb")) == NULL) {
 		FILE_OP_ERROR(tmp, "fopen");
 		return -1;
 	}
 	if (change_file_mode_rw(tmpfp, tmp) < 0) {
 		FILE_OP_ERROR(tmp, "chmod");
-		g_warning(_("can't change file mode\n"));
+		g_warning("can't change file mode\n");
 	}
 	
 	fprintf(tmpfp, "From: %s\r\n", msginfo->from);
@@ -945,3 +743,363 @@ gint news_cancel_article(Folder * folder, MsgInfo * msginfo)
 
 	return 0;
 }
+
+static gchar *news_folder_get_path(Folder *folder)
+{
+	gchar *folder_path;
+
+        g_return_val_if_fail(folder->account != NULL, NULL);
+
+        folder_path = g_strconcat(get_news_cache_dir(),
+                                  G_DIR_SEPARATOR_S,
+                                  folder->account->nntp_server,
+                                  NULL);
+	return folder_path;
+}
+
+gchar *news_item_get_path(Folder *folder, FolderItem *item)
+{
+	gchar *folder_path, *path;
+
+	g_return_val_if_fail(folder != NULL, NULL);
+	g_return_val_if_fail(item != NULL, NULL);
+	folder_path = news_folder_get_path(folder);
+
+        g_return_val_if_fail(folder_path != NULL, NULL);
+        if (folder_path[0] == G_DIR_SEPARATOR) {
+                if (item->path)
+                        path = g_strconcat(folder_path, G_DIR_SEPARATOR_S,
+                                           item->path, NULL);
+                else
+                        path = g_strdup(folder_path);
+        } else {
+                if (item->path)
+                        path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
+                                           folder_path, G_DIR_SEPARATOR_S,
+                                           item->path, NULL);
+                else
+                        path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
+                                           folder_path, NULL);
+        }
+        g_free(folder_path);
+
+	return path;
+}
+
+gint news_get_num_list(Folder *folder, FolderItem *item, GSList **msgnum_list, gboolean *old_uids_valid)
+{
+	NNTPSession *session;
+	gint i, ok, num, first, last, nummsgs = 0;
+	gchar *dir;
+
+	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->folder != NULL, -1);
+	g_return_val_if_fail(FOLDER_CLASS(folder) == &news_class, -1);
+
+	session = news_session_get(folder);
+	g_return_val_if_fail(session != NULL, -1);
+
+	*old_uids_valid = TRUE;
+
+	ok = news_select_group(session, item->path, &num, &first, &last);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't set group: %s\n"), item->path);
+		return -1;
+	}
+
+	dir = news_folder_get_path(folder);
+	if (num <= 0)
+		remove_all_numbered_files(dir);
+	else if (last < first)
+		log_warning(_("invalid article range: %d - %d\n"),
+			    first, last);
+	else {
+		for (i = first; i <= last; i++) {
+			*msgnum_list = g_slist_prepend(*msgnum_list, 
+						       GINT_TO_POINTER(i));
+			nummsgs++;
+		}
+		debug_print("removing old messages from %d to %d in %s\n",
+			    first, last, dir);
+		remove_numbered_files(dir, 1, first - 1);
+	}
+	g_free(dir);
+
+	return nummsgs;
+}
+
+#define READ_TO_LISTEND(hdr) \
+	while (!(buf[0] == '.' && buf[1] == '\r')) { \
+		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) { \
+			log_warning(_("error occurred while getting %s.\n"), hdr); \
+			return msginfo; \
+		} \
+	}
+
+MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
+{
+	NNTPSession *session;
+	MsgInfo *msginfo = NULL;
+	gchar buf[NNTPBUFSIZE];
+	gint ok;
+
+	session = news_session_get(folder);
+	g_return_val_if_fail(session != NULL, NULL);
+	g_return_val_if_fail(item != NULL, NULL);
+	g_return_val_if_fail(item->folder != NULL, NULL);
+	g_return_val_if_fail(FOLDER_CLASS(item->folder) == &news_class, NULL);
+
+	log_message(_("getting xover %d in %s...\n"),
+		    num, item->path);
+	ok = nntp_xover(session, num, num);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't get xover\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
+		return NULL;
+	}
+	
+	if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
+		log_warning(_("error occurred while getting xover.\n"));
+		return NULL;
+	}
+	
+	msginfo = news_parse_xover(buf);
+	if (!msginfo) {
+		log_warning(_("invalid xover line: %s\n"), buf);
+	}
+
+	READ_TO_LISTEND("xover");
+
+	if(!msginfo)
+		return NULL;
+	
+	msginfo->folder = item;
+	msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
+	msginfo->flags.tmp_flags = MSG_NEWS;
+	msginfo->newsgroups = g_strdup(item->path);
+
+	ok = nntp_xhdr(session, "to", num, num);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't get xhdr\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
+		return msginfo;
+	}
+
+	if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
+		log_warning(_("error occurred while getting xhdr.\n"));
+		return msginfo;
+	}
+
+	msginfo->to = news_parse_xhdr(buf, msginfo);
+
+	READ_TO_LISTEND("xhdr (to)");
+
+	ok = nntp_xhdr(session, "cc", num, num);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't get xhdr\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
+		return msginfo;
+	}
+
+	if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
+		log_warning(_("error occurred while getting xhdr.\n"));
+		return msginfo;
+	}
+
+	msginfo->cc = news_parse_xhdr(buf, msginfo);
+
+	READ_TO_LISTEND("xhdr (cc)");
+
+	return msginfo;
+}
+
+static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *item, guint begin, guint end)
+{
+	gchar buf[NNTPBUFSIZE];
+	GSList *newlist = NULL;
+	GSList *llast = NULL;
+	MsgInfo *msginfo;
+	guint count = 0, lines = (end - begin + 2) * 3;
+	gint ok;
+
+	g_return_val_if_fail(session != NULL, NULL);
+	g_return_val_if_fail(item != NULL, NULL);
+
+	log_message(_("getting xover %d - %d in %s...\n"),
+		    begin, end, item->path);
+	ok = nntp_xover(session, begin, end);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't get xover\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
+		return NULL;
+	}
+
+	for (;;) {
+		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
+			log_warning(_("error occurred while getting xover.\n"));
+			return newlist;
+		}
+		count++;
+		progressindicator_set_percentage
+			(PROGRESS_TYPE_NETWORK,
+			 session->fetch_base_percentage +
+			 (((gfloat) count) / ((gfloat) lines)) * session->fetch_total_percentage);
+
+		if (buf[0] == '.' && buf[1] == '\r') break;
+
+		msginfo = news_parse_xover(buf);
+		if (!msginfo) {
+			log_warning(_("invalid xover line: %s\n"), buf);
+			continue;
+		}
+
+		msginfo->folder = item;
+		msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
+		msginfo->flags.tmp_flags = MSG_NEWS;
+		msginfo->newsgroups = g_strdup(item->path);
+
+		if (!newlist)
+			llast = newlist = g_slist_append(newlist, msginfo);
+		else {
+			llast = g_slist_append(llast, msginfo);
+			llast = llast->next;
+		}
+	}
+
+	ok = nntp_xhdr(session, "to", begin, end);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't get xhdr\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
+		return newlist;
+	}
+
+	llast = newlist;
+
+	for (;;) {
+		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
+			log_warning(_("error occurred while getting xhdr.\n"));
+			return newlist;
+		}
+		count++;
+		progressindicator_set_percentage
+			(PROGRESS_TYPE_NETWORK,
+			 session->fetch_base_percentage +
+			 (((gfloat) count) / ((gfloat) lines)) * session->fetch_total_percentage);
+
+		if (buf[0] == '.' && buf[1] == '\r') break;
+		if (!llast) {
+			g_warning("llast == NULL\n");
+			continue;
+		}
+
+		msginfo = (MsgInfo *)llast->data;
+		msginfo->to = news_parse_xhdr(buf, msginfo);
+
+		llast = llast->next;
+	}
+
+	ok = nntp_xhdr(session, "cc", begin, end);
+	if (ok != NN_SUCCESS) {
+		log_warning(_("can't get xhdr\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
+		return newlist;
+	}
+
+	llast = newlist;
+
+	for (;;) {
+		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
+			log_warning(_("error occurred while getting xhdr.\n"));
+			return newlist;
+		}
+		count++;
+		progressindicator_set_percentage
+			(PROGRESS_TYPE_NETWORK,
+			 session->fetch_base_percentage +
+			 (((gfloat) count) / ((gfloat) lines)) * session->fetch_total_percentage);
+
+		if (buf[0] == '.' && buf[1] == '\r') break;
+		if (!llast) {
+			g_warning("llast == NULL\n");
+			continue;
+		}
+
+		msginfo = (MsgInfo *)llast->data;
+		msginfo->cc = news_parse_xhdr(buf, msginfo);
+
+		llast = llast->next;
+	}
+
+	return newlist;
+}
+
+GSList *news_get_msginfos(Folder *folder, FolderItem *item, GSList *msgnum_list)
+{
+	NNTPSession *session;
+	GSList *elem, *msginfo_list = NULL, *tmp_msgnum_list, *tmp_msginfo_list;
+	guint first, last, next;
+	guint tofetch, fetched;
+	
+	g_return_val_if_fail(folder != NULL, NULL);
+	g_return_val_if_fail(FOLDER_CLASS(folder) == &news_class, NULL);
+	g_return_val_if_fail(msgnum_list != NULL, NULL);
+	g_return_val_if_fail(item != NULL, NULL);
+	
+	session = news_session_get(folder);
+	g_return_val_if_fail(session != NULL, NULL);
+
+	tmp_msgnum_list = g_slist_copy(msgnum_list);
+	tmp_msgnum_list = g_slist_sort(tmp_msgnum_list, g_int_compare);
+
+	progressindicator_start(PROGRESS_TYPE_NETWORK);
+	tofetch = g_slist_length(tmp_msgnum_list);
+	fetched = 0;
+
+	first = GPOINTER_TO_INT(tmp_msgnum_list->data);
+	last = first;
+	for(elem = g_slist_next(tmp_msgnum_list); elem != NULL; elem = g_slist_next(elem)) {
+		next = GPOINTER_TO_INT(elem->data);
+		if(next != (last + 1)) {
+			session->fetch_base_percentage = ((gfloat) fetched) / ((gfloat) tofetch);
+			session->fetch_total_percentage = ((gfloat) (last - first + 1)) / ((gfloat) tofetch);
+			tmp_msginfo_list = news_get_msginfos_for_range(session, item, first, last);
+			msginfo_list = g_slist_concat(msginfo_list, tmp_msginfo_list);
+			fetched = last - first + 1;
+			first = next;
+		}
+		last = next;
+	}
+	session->fetch_base_percentage = ((gfloat) fetched) / ((gfloat) tofetch);
+	session->fetch_total_percentage = ((gfloat) (last - first + 1)) / ((gfloat) tofetch);
+	tmp_msginfo_list = news_get_msginfos_for_range(session, item, first, last);
+	msginfo_list = g_slist_concat(msginfo_list, tmp_msginfo_list);
+
+	g_slist_free(tmp_msgnum_list);
+	
+	progressindicator_stop(PROGRESS_TYPE_NETWORK);
+
+	return msginfo_list;
+}
+
+gboolean news_scan_required(Folder *folder, FolderItem *item)
+{
+	return TRUE;
+}
