@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include <errno.h>
 
 #include "intl.h"
-#include "prefs.h"
+#include "prefs_gtk.h"
 #include "prefs_display_header.h"
 #include "prefs_common.h"
 #include "manage_window.h"
@@ -175,7 +175,7 @@ static void prefs_display_header_create(void)
 
 	gchar *title[1];
 
-	debug_print(_("Creating display header setting window...\n"));
+	debug_print("Creating display header setting window...\n");
 
 	window = gtk_window_new (GTK_WINDOW_DIALOG);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
@@ -198,7 +198,7 @@ static void prefs_display_header_create(void)
 	gtk_widget_grab_default (ok_btn);
 
 	gtk_window_set_title (GTK_WINDOW(window),
-			      _("Display header setting"));
+			      _("Displayed header configuration"));
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 	gtk_signal_connect (GTK_OBJECT(window), "delete_event",
 			    GTK_SIGNAL_FUNC(prefs_display_header_deleted),
@@ -277,7 +277,7 @@ static void prefs_display_header_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), reg_btn, FALSE, TRUE, 0);
 	gtk_signal_connect (GTK_OBJECT (reg_btn), "clicked",
 			    GTK_SIGNAL_FUNC (prefs_display_header_register_cb),
-			    FALSE);
+			    GINT_TO_POINTER(FALSE));
 	del_btn = gtk_button_new_with_label (_("Delete"));
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), del_btn, FALSE, TRUE, 0);
@@ -333,13 +333,13 @@ static void prefs_display_header_create(void)
 	gtk_signal_connect (GTK_OBJECT (reg_btn), "clicked",
 			    GTK_SIGNAL_FUNC
 			    (prefs_display_header_register_cb),
-			    (void *) TRUE);
+			    GINT_TO_POINTER(TRUE));
 	del_btn = gtk_button_new_with_label (_("Delete"));
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), del_btn, FALSE, TRUE, 0);
 	gtk_signal_connect (GTK_OBJECT (del_btn), "clicked",
 			    GTK_SIGNAL_FUNC (prefs_display_header_delete_cb),
-			    (void *) hidden_headers_clist);
+			    hidden_headers_clist);
 
 	PACK_CHECK_BUTTON (btn_hbox, checkbtn_other_headers,
 			   _("Show all unspecified headers"));
@@ -367,11 +367,11 @@ void prefs_display_header_read_config(void)
 	gchar buf[PREFSBUFSIZE];
 	DisplayHeaderProp *dp;
 
-	debug_print(_("Reading configuration for displaying headers...\n"));
+	debug_print("Reading configuration for displaying headers...\n");
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			     DISPLAY_HEADER_RC, NULL);
-	if ((fp = fopen(rcpath, "r")) == NULL) {
+	if ((fp = fopen(rcpath, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 		g_free(rcpath);
 		prefs_common.disphdr_list = NULL;
@@ -405,13 +405,13 @@ void prefs_display_header_write_config(void)
 	PrefFile *pfile;
 	GSList *cur;
 
-	debug_print(_("Writing configuration for displaying headers...\n"));
+	debug_print("Writing configuration for displaying headers...\n");
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			     DISPLAY_HEADER_RC, NULL);
 
 	if ((pfile = prefs_write_open(rcpath)) == NULL) {
-		g_warning(_("failed to write configuration to file\n"));
+		g_warning("failed to write configuration to file\n");
 		g_free(rcpath);
 		return;
 	}
@@ -425,7 +425,7 @@ void prefs_display_header_write_config(void)
 		if (fputs(dpstr, pfile->fp) == EOF ||
 		    fputc('\n', pfile->fp) == EOF) {
 			FILE_OP_ERROR(rcpath, "fputs || fputc");
-			prefs_write_close_revert(pfile);
+			prefs_file_close_revert(pfile);
 			g_free(rcpath);
 			g_free(dpstr);
 			return;
@@ -435,8 +435,8 @@ void prefs_display_header_write_config(void)
 
 	g_free(rcpath);
 
-	if (prefs_write_close(pfile) < 0) {
-		g_warning(_("failed to write configuration to file\n"));
+	if (prefs_file_close(pfile) < 0) {
+		g_warning("failed to write configuration to file\n");
 		return;
 	}
 }
@@ -558,9 +558,7 @@ static gint prefs_display_header_clist_set_row(gboolean hidden)
 static void prefs_display_header_register_cb(GtkButton *btn,
 					     gpointer hidden_data)
 {
-	gboolean hidden = (gboolean)hidden_data;
-
-	prefs_display_header_clist_set_row(hidden);
+	prefs_display_header_clist_set_row(GPOINTER_TO_INT(hidden_data));
 }
 
 static void prefs_display_header_delete_cb(GtkButton *btn, gpointer clist_data)
