@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +36,70 @@
 
 #define BUFFSIZE	8192
 
+typedef char *(*getlinefunc)(char *, int, void *);
+typedef int (*peekcharfunc)(void *);
+typedef int (*getcharfunc)(void *);
+typedef gint (*get_one_field_func)(gchar *, gint, void *, HeaderEntry[]);
+
+static gint string_get_one_field(gchar *buf, gint len, char **str,
+				 HeaderEntry hentry[]);
+
+static char *string_getline(char *buf, int len, char **str);
+static int string_peekchar(char **str);
+static int file_peekchar(FILE *fp);
+static gint generic_get_one_field(gchar *buf, gint len, void *data,
+				  HeaderEntry hentry[],
+				  getlinefunc getline, 
+				  peekcharfunc peekchar,
+				  gboolean unfold);
+static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
+			     gboolean full, gboolean decrypted);
+
+
 gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 			      HeaderEntry hentry[])
+{
+	return generic_get_one_field(buf, len, fp, hentry,
+				     (getlinefunc)fgets, (peekcharfunc)file_peekchar,
+				     TRUE);
+}
+
+static gint string_get_one_field(gchar *buf, gint len, char **str,
+				 HeaderEntry hentry[])
+{
+	return generic_get_one_field(buf, len, str, hentry,
+				     (getlinefunc)string_getline,
+				     (peekcharfunc)string_peekchar,
+				     TRUE);
+}
+
+static char *string_getline(char *buf, int len, char **str)
+{
+	if (!**str)
+		return NULL;
+
+	for (; **str && len > 1; --len)
+		if ((*buf++ = *(*str)++) == '\n')
+		    break;
+	*buf = '\0';
+
+	return buf;
+}
+
+static int string_peekchar(char **str)
+{
+	return **str;
+}
+
+static int file_peekchar(FILE *fp)
+{
+	return ungetc(getc(fp), fp);
+}
+
+static gint generic_get_one_field(gchar *buf, gint len, void *data,
+			  HeaderEntry *hentry,
+			  getlinefunc getline, peekcharfunc peekchar,
+			  gboolean unfold)
 {
 	gint nexthead;
 	gint hnum = 0;
@@ -47,7 +109,7 @@ gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 		/* skip non-required headers */
 		do {
 			do {
-				if (fgets(buf, len, fp) == NULL)
+				if (getline(buf, len, data) == NULL)
 					return -1;
 				if (buf[0] == '\r' || buf[0] == '\n')
 					return -1;
@@ -56,88 +118,53 @@ gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 			for (hp = hentry, hnum = 0; hp->name != NULL;
 			     hp++, hnum++) {
 				if (!strncasecmp(hp->name, buf,
-						 strlen(hp->name))) {
+						 strlen(hp->name)))
 					break;
-				}
 			}
 		} while (hp->name == NULL);
 	} else {
-		if (fgets(buf, len, fp) == NULL) return -1;
+		if (getline(buf, len, data) == NULL) return -1;
 		if (buf[0] == '\r' || buf[0] == '\n') return -1;
 	}
 
-	/* unfold the specified folded line */
-	if (hp && hp->unfold) {
-		gboolean folded = FALSE;
-		gchar *bufp = buf + strlen(buf);
-
-		while (1) {
-			nexthead = fgetc(fp);
-
-			/* folded */
-			if (nexthead == ' ' || nexthead == '\t')
-				folded = TRUE;
-			else if (nexthead == EOF)
-				break;
-			else if (folded == TRUE) {
-				/* concatenate next line */
-				if ((len - (bufp - buf)) <= 2) break;
-
-				/* replace return code on the tail end
-				   with space */
-				*(bufp - 1) = ' ';
-				*bufp++ = nexthead;
-				*bufp = '\0';
-				if (nexthead == '\r' || nexthead == '\n') {
-					folded = FALSE;
-					continue;
-				}
-				if (fgets(bufp, len - (bufp - buf), fp)
-				    == NULL) break;
-				bufp += strlen(bufp);
-
-				folded = FALSE;
-			} else {
-				ungetc(nexthead, fp);
-				break;
-			}
-		}
-
-		/* remove trailing return code */
-		strretchomp(buf);
-
-		return hnum;
-	}
-
+	/* unfold line */
 	while (1) {
-		nexthead = fgetc(fp);
+		nexthead = peekchar(data);
+		/* ([*WSP CRLF] 1*WSP) */
 		if (nexthead == ' ' || nexthead == '\t') {
-			size_t buflen = strlen(buf);
+			size_t buflen;
+			/* trim previous trailing \n if requesting one header or
+			 * unfolding was requested */
+			if ((!hentry && unfold) || (hp && hp->unfold))
+				strretchomp(buf);
 
+			buflen = strlen(buf);
+			
 			/* concatenate next line */
 			if ((len - buflen) > 2) {
-				gchar *p = buf + buflen;
-
-				*p++ = nexthead;
-				*p = '\0';
-				buflen++;
-				if (fgets(p, len - buflen, fp) == NULL)
+				if (getline(buf + buflen, len - buflen, data) == NULL)
 					break;
 			} else
 				break;
 		} else {
-			if (nexthead != EOF)
-				ungetc(nexthead, fp);
+			/* remove trailing new line */
+			strretchomp(buf);
 			break;
 		}
 	}
 
-	/* remove trailing return code */
-	strretchomp(buf);
-
 	return hnum;
 }
 
+gint procheader_get_one_field_asis(gchar *buf, gint len, FILE *fp)
+{
+	return generic_get_one_field(buf, len, fp, NULL,
+				     (getlinefunc)fgets, 
+				     (peekcharfunc)file_peekchar,
+				     FALSE);
+}
+
+#if 0
 gchar *procheader_get_unfolded_line(gchar *buf, gint len, FILE *fp)
 {
 	gboolean folded = FALSE;
@@ -148,6 +175,11 @@ gchar *procheader_get_unfolded_line(gchar *buf, gint len, FILE *fp)
 	if (buf[0] == '\r' || buf[0] == '\n') return NULL;
 	bufp = buf + strlen(buf);
 
+	for (; bufp > buf &&
+	     (*(bufp - 1) == '\n' || *(bufp - 1) == '\r');
+	     bufp--)
+		*(bufp - 1) = '\0';
+
 	while (1) {
 		nexthead = fgetc(fp);
 
@@ -157,22 +189,29 @@ gchar *procheader_get_unfolded_line(gchar *buf, gint len, FILE *fp)
 		else if (nexthead == EOF)
 			break;
 		else if (folded == TRUE) {
-			/* concatenate next line */
-			if ((len - (bufp - buf)) <= 2) break;
-
-			/* replace return code on the tail end
-			   with space */
-			*(bufp - 1) = ' ';
-			*bufp++ = nexthead;
-			*bufp = '\0';
 			if (nexthead == '\r' || nexthead == '\n') {
 				folded = FALSE;
 				continue;
 			}
+
+			if ((len - (bufp - buf)) <= 2) break;
+
+			/* replace return code on the tail end
+			   with space */
+			*bufp++ = ' ';
+			*bufp++ = nexthead;
+			*bufp = '\0';
+
+			/* concatenate next line */
 			if (fgets(bufp, len - (bufp - buf), fp)
 			    == NULL) break;
 			bufp += strlen(bufp);
 
+			for (; bufp > buf &&
+			     (*(bufp - 1) == '\n' || *(bufp - 1) == '\r');
+			     bufp--)
+				*(bufp - 1) = '\0';
+
 			folded = FALSE;
 		} else {
 			ungetc(nexthead, fp);
@@ -185,13 +224,15 @@ gchar *procheader_get_unfolded_line(gchar *buf, gint len, FILE *fp)
 
 	return buf;
 }
+#endif
 
+#if 0
 GSList *procheader_get_header_list_from_file(const gchar *file)
 {
 	FILE *fp;
 	GSList *hlist;
 
-	if ((fp = fopen(file, "r")) == NULL) {
+	if ((fp = fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return NULL;
 	}
@@ -204,15 +245,14 @@ GSList *procheader_get_header_list_from_file(const gchar *file)
 
 GSList *procheader_get_header_list(FILE *fp)
 {
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
-	gchar *p;
+	gchar buf[BUFFSIZE];
 	GSList *hlist = NULL;
 	Header *header;
 
 	g_return_val_if_fail(fp != NULL, NULL);
 
 	while (procheader_get_unfolded_line(buf, sizeof(buf), fp) != NULL) {
-		if (header = procheader_parse_header(buf))
+		if ((header = procheader_parse_header(buf)) != NULL)
 			hlist = g_slist_append(hlist, header);
 		/*
 		if (*buf == ':') continue;
@@ -234,11 +274,12 @@ GSList *procheader_get_header_list(FILE *fp)
 
 	return hlist;
 }
+#endif
 
+#if 0
 GPtrArray *procheader_get_header_array(FILE *fp)
 {
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
-	gchar *p;
+	gchar buf[BUFFSIZE];
 	GPtrArray *headers;
 	Header *header;
 
@@ -247,7 +288,7 @@ GPtrArray *procheader_get_header_array(FILE *fp)
 	headers = g_ptr_array_new();
 
 	while (procheader_get_unfolded_line(buf, sizeof(buf), fp) != NULL) {
-		if (header = procheader_parse_header(buf))
+		if ((header = procheader_parse_header(buf)) != NULL)
 			g_ptr_array_add(headers, header);
 		/*
 		if (*buf == ':') continue;
@@ -269,11 +310,11 @@ GPtrArray *procheader_get_header_array(FILE *fp)
 
 	return headers;
 }
+#endif
 
 GPtrArray *procheader_get_header_array_asis(FILE *fp)
 {
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
-	gchar *p;
+	gchar buf[BUFFSIZE];
 	GPtrArray *headers;
 	Header *header;
 
@@ -281,8 +322,8 @@ GPtrArray *procheader_get_header_array_asis(FILE *fp)
 
 	headers = g_ptr_array_new();
 
-	while (procheader_get_one_field(buf, sizeof(buf), fp, NULL) != -1) {
-		if (header = procheader_parse_header(buf))
+	while (procheader_get_one_field_asis(buf, sizeof(buf), fp) != -1) {
+		if ((header = procheader_parse_header(buf)) != NULL)
 			g_ptr_array_add(headers, header);
 			/*
 		if (*buf == ':') continue;
@@ -304,6 +345,7 @@ GPtrArray *procheader_get_header_array_asis(FILE *fp)
 	return headers;
 }
 
+#if 0
 void procheader_header_list_destroy(GSList *hlist)
 {
 	Header *header;
@@ -314,6 +356,7 @@ void procheader_header_list_destroy(GSList *hlist)
 		hlist = g_slist_remove(hlist, header);
 	}
 }
+#endif
 
 void procheader_header_array_destroy(GPtrArray *harray)
 {
@@ -382,7 +425,10 @@ Header * procheader_parse_header(gchar * buf)
 			p++;
 			while (*p == ' ' || *p == '\t') p++;
 			conv_unmime_header(tmp, sizeof(tmp), p, NULL);
-			header->body = g_strdup(tmp);
+			if(tmp == NULL) 
+				header->body = g_strdup(p);
+			else	
+				header->body = g_strdup(tmp);
 			return header;
 		}
 	}
@@ -416,6 +462,28 @@ void procheader_get_header_fields(FILE *fp, HeaderEntry hentry[])
 	}
 }
 
+MsgInfo *procheader_parse_file(const gchar *file, MsgFlags flags,
+			       gboolean full, gboolean decrypted)
+{
+	FILE *fp;
+	MsgInfo *msginfo;
+
+	if ((fp = fopen(file, "rb")) == NULL) {
+		FILE_OP_ERROR(file, "fopen");
+		return NULL;
+	}
+
+	msginfo = procheader_parse_stream(fp, flags, full, decrypted);
+	fclose(fp);
+	return msginfo;
+}
+
+MsgInfo *procheader_parse_str(const gchar *str, MsgFlags flags, gboolean full,
+			      gboolean decrypted)
+{
+	return parse_stream(&str, TRUE, flags, full, decrypted);
+}
+
 enum
 {
 	H_DATE		= 0,
@@ -437,69 +505,55 @@ enum
 	H_RETURN_RECEIPT_TO = 16
 };
 
-MsgInfo *procheader_parse(const gchar *file, MsgFlags flags,
-			  gboolean full, gboolean decrypted)
+static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
+				   {"From:",		NULL, TRUE},
+				   {"To:",		NULL, TRUE},
+				   {"Cc:",		NULL, TRUE},
+				   {"Newsgroups:",	NULL, TRUE},
+				   {"Subject:",		NULL, TRUE},
+				   {"Message-Id:",	NULL, FALSE},
+				   {"References:",	NULL, FALSE},
+				   {"In-Reply-To:",	NULL, FALSE},
+				   {"Content-Type:",	NULL, FALSE},
+				   {"Seen:",		NULL, FALSE},
+				   {"Status:",          NULL, FALSE},
+				   {"X-Status:",        NULL, FALSE},
+				   {"From ",		NULL, FALSE},
+				   {"X-Face:",		NULL, FALSE},
+				   {"Disposition-Notification-To:", NULL, FALSE},
+				   {"Return-Receipt-To:", NULL, FALSE},
+				   {NULL,		NULL, FALSE}};
+
+static HeaderEntry hentry_short[] = {{"Date:",		NULL, FALSE},
+				    {"From:",		NULL, TRUE},
+				    {"To:",		NULL, TRUE},
+				    {"Cc:",		NULL, TRUE},
+				    {"Newsgroups:",	NULL, TRUE},
+				    {"Subject:",	NULL, TRUE},
+				    {"Message-Id:",	NULL, FALSE},
+				    {"References:",	NULL, FALSE},
+				    {"In-Reply-To:",	NULL, FALSE},
+				    {"Content-Type:",	NULL, FALSE},
+				    {"Seen:",		NULL, FALSE},
+				    {"Status:",		NULL, FALSE},
+				    {"X-Status:",	NULL, FALSE},
+				    {"From ",		NULL, FALSE},
+				    {NULL,		NULL, FALSE}};
+
+HeaderEntry* procheader_get_headernames(gboolean full)
 {
-	FILE *fp;
-	MsgInfo *msginfo;
-
-	if ((fp = fopen(file, "r")) == NULL) {
-		FILE_OP_ERROR(file, "fopen");
-		return NULL;
-	}
-
-	msginfo = procheader_file_parse(fp, flags, full, decrypted);
-
-	fclose(fp);
-
-	return msginfo;
+	return full ? hentry_full : hentry_short;
 }
 
-
-/* FIXME: we should not allow headers in messages to change the sylpheed marks
- * so we're currently disabling setting any MsgFlags when detecting X-Seen,
- * Seen, X-Status, Status. See macro ALLOW_HEADER_HINT */  
-
-/* #define ALLOW_HEADER_HINT */
-
-MsgInfo *procheader_file_parse(FILE * fp, MsgFlags flags,
-			       gboolean full, gboolean decrypted)
+MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full,
+				 gboolean decrypted)
 {
-	static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
-					   {"From:",		NULL, TRUE},
-					   {"To:",		NULL, TRUE},
-					   {"Cc:",		NULL, TRUE},
-					   {"Newsgroups:",	NULL, TRUE},
-					   {"Subject:",		NULL, TRUE},
-					   {"Message-Id:",	NULL, FALSE},
-					   {"References:",	NULL, FALSE},
-					   {"In-Reply-To:",	NULL, FALSE},
-					   {"Content-Type:",	NULL, FALSE},
-					   {"Seen:",		NULL, FALSE},
-					   {"Status:",          NULL, FALSE},
-					   {"X-Status:",        NULL, FALSE},
-					   {"From ",		NULL, FALSE},
-					   {"X-Face:",		NULL, FALSE},
-					   {"Disposition-Notification-To:", NULL, FALSE},
-					   {"Return-Receipt-To:", NULL, FALSE},
-					   {NULL,		NULL, FALSE}};
+	return parse_stream(fp, FALSE, flags, full, decrypted);
+}
 
-	static HeaderEntry hentry_short[] = {{"Date:",		NULL, FALSE},
-					    {"From:",		NULL, TRUE},
-					    {"To:",		NULL, TRUE},
-					    {"Cc:",		NULL, TRUE},
-					    {"Newsgroups:",	NULL, TRUE},
-					    {"Subject:",	NULL, TRUE},
-					    {"Message-Id:",	NULL, FALSE},
-					    {"References:",	NULL, FALSE},
-					    {"In-Reply-To:",	NULL, FALSE},
-					    {"Content-Type:",	NULL, FALSE},
-					    {"Seen:",		NULL, FALSE},
-					    {"Status:",		NULL, FALSE},
-					    {"X-Status:",	NULL, FALSE},
-					    {"From ",		NULL, FALSE},
-					    {NULL,		NULL, FALSE}};
-	
+static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
+			     gboolean full, gboolean decrypted)
+{
 	MsgInfo *msginfo;
 	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
 	gchar *reference = NULL;
@@ -507,15 +561,18 @@ MsgInfo *procheader_file_parse(FILE * fp, MsgFlags flags,
 	gchar *hp;
 	HeaderEntry *hentry;
 	gint hnum;
+	get_one_field_func get_one_field =
+		isstring ? (get_one_field_func)string_get_one_field
+			 : (get_one_field_func)procheader_get_one_field;
 
-	hentry = full ? hentry_full : hentry_short;
+	hentry = procheader_get_headernames(full);
 
-	if (MSG_IS_QUEUED(flags)) {
-		while (fgets(buf, sizeof(buf), fp) != NULL)
-			if (buf[0] == '\r' || buf[0] == '\n') break;
+	if (MSG_IS_QUEUED(flags) || MSG_IS_DRAFT(flags)) {
+		while (get_one_field(buf, sizeof(buf), data, NULL) != -1)
+			; /* loop */
 	}
 
-	msginfo = g_new0(MsgInfo, 1);
+	msginfo = procmsg_msginfo_new();
 	
 	if (flags.tmp_flags || flags.perm_flags) 
 		msginfo->flags = flags;
@@ -527,10 +584,9 @@ MsgInfo *procheader_file_parse(FILE * fp, MsgFlags flags,
 
 	msginfo->inreplyto = NULL;
 
-	while ((hnum = procheader_get_one_field(buf, sizeof(buf), fp, hentry))
+	while ((hnum = get_one_field(buf, sizeof(buf), data, hentry))
 	       != -1) {
 		hp = buf + strlen(hentry[hnum].name);
-
 		while (*hp == ' ' || *hp == '\t') hp++;
 
 		switch (hnum) {
@@ -573,7 +629,7 @@ MsgInfo *procheader_file_parse(FILE * fp, MsgFlags flags,
 					g_strconcat(p, ",", hp, NULL);
 				g_free(p);
 			} else
-				msginfo->newsgroups = g_strdup(buf + 12);
+				msginfo->newsgroups = g_strdup(hp);
 			break;
 		case H_SUBJECT:
 			if (msginfo->subject) break;
@@ -603,16 +659,24 @@ MsgInfo *procheader_file_parse(FILE * fp, MsgFlags flags,
 			break;
 		case H_CONTENT_TYPE:
 			if (decrypted) {
-				if (!strncasecmp(hp, "multipart", 9) &&
-				    strncasecmp(hp, "multipart/signed", 16)) {
-					MSG_SET_TMP_FLAGS(msginfo->flags,
+				if (!strncasecmp(hp, "multipart", 9)) {
+					if (strncasecmp(hp, "multipart/signed", 16)) {
+						MSG_SET_TMP_FLAGS(msginfo->flags,
 							  MSG_MIME);
+					} else {
+						MSG_SET_TMP_FLAGS(msginfo->flags,
+							  MSG_SIGNED);
+					}
 				}
 			}
 			else if (!strncasecmp(hp, "multipart/encrypted", 19)) {
 				MSG_SET_TMP_FLAGS(msginfo->flags,
 						  MSG_ENCRYPTED);
-			}
+			} 
+			else if (!strncasecmp(hp, "multipart", 9) &&
+				   !strncasecmp(hp, "multipart/signed", 16)) {
+				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_SIGNED);
+			} 
 			else if (!strncasecmp(hp, "multipart", 9))
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MIME);
 			break;
@@ -629,12 +693,10 @@ MsgInfo *procheader_file_parse(FILE * fp, MsgFlags flags,
 		case H_DISPOSITION_NOTIFICATION_TO:
 			if (msginfo->dispositionnotificationto) break;
 			msginfo->dispositionnotificationto = g_strdup(hp);
-			MSG_SET_PERM_FLAGS(msginfo->flags, MSG_RETRCPT_PENDING);	
 			break;
 		case H_RETURN_RECEIPT_TO:
 			if (msginfo->returnreceiptto) break;
 			msginfo->returnreceiptto = g_strdup(hp);
-			MSG_SET_PERM_FLAGS(msginfo->flags, MSG_RETRCPT_PENDING);	
 			break;
 #ifdef ALLOW_HEADER_HINT			
 		case H_STATUS:
@@ -702,7 +764,48 @@ gchar *procheader_get_fromname(const gchar *str)
 	return name;
 }
 
-time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
+static gint procheader_scan_date_string(const gchar *str,
+					gchar *weekday, gint *day,
+					gchar *month, gint *year,
+					gint *hh, gint *mm, gint *ss,
+					gchar *zone)
+{
+	gint result;
+
+	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d %5s",
+			weekday, day, month, year, hh, mm, ss, zone);
+	if (result == 8) return 0;
+
+	result = sscanf(str, "%3s,%d %9s %d %2d:%2d:%2d %5s",
+			weekday, day, month, year, hh, mm, ss, zone);
+	if (result == 8) return 0;
+
+	result = sscanf(str, "%d %9s %d %2d:%2d:%2d %5s",
+			day, month, year, hh, mm, ss, zone);
+	if (result == 7) return 0;
+
+	*zone = '\0';
+	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d",
+			weekday, day, month, year, hh, mm, ss);
+	if (result == 7) return 0;
+
+	*ss = 0;
+	result = sscanf(str, "%10s %d %9s %d %2d:%2d %5s",
+			weekday, day, month, year, hh, mm, zone);
+	if (result == 7) return 0;
+
+	result = sscanf(str, "%d %9s %d %2d:%2d %5s",
+			day, month, year, hh, mm, zone);
+	if (result == 6) return 0;
+
+	return -1;
+}
+
+/*
+ * Hiro, most UNIXen support this function:
+ * http://www.mcsr.olemiss.edu/cgi-bin/man-cgi?getdate
+ */
+gboolean procheader_date_parse_to_tm(const gchar *src, struct tm *t, char *zone)
 {
 	static gchar monthstr[] = "JanFebMarAprMayJunJulAugSepOctNovDec";
 	gchar weekday[11];
@@ -710,35 +813,18 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	gchar month[10];
 	gint year;
 	gint hh, mm, ss;
-	gchar zone[6];
-	gint result;
 	GDateMonth dmonth;
-	struct tm t;
 	gchar *p;
-	time_t timer;
 
-	/* parsing date field... */
-	result = sscanf(src, "%10s %d %9s %d %2d:%2d:%2d %5s",
-			weekday, &day, month, &year, &hh, &mm, &ss, zone);
-	if (result != 8) {
-		result = sscanf(src, "%d %9s %d %2d:%2d:%2d %5s",
-				&day, month, &year, &hh, &mm, &ss, zone);
-		if (result != 7) {
-			ss = 0;
-			result = sscanf(src, "%10s %d %9s %d %2d:%2d %5s",
-					weekday, &day, month, &year, &hh, &mm, zone);
-			if (result != 7) {
-				result = sscanf(src, "%d %9s %d %2d:%2d %5s",
-						&day, month, &year, &hh, &mm,
-						zone);
-				if (result != 6) {
-					g_warning("Invalid date: %s\n", src);
-					if (dest && len > 0)
-						strncpy2(dest, src, len);
-					return 0;
-				}
-			}
-		}
+	if (!t)
+		return FALSE;
+	
+	memset(t, 0, sizeof *t);	
+
+	if (procheader_scan_date_string(src, weekday, &day, month, &year,
+					&hh, &mm, &ss, zone) < 0) {
+		g_warning("Invalid date: %s\n", src);
+		return FALSE;
 	}
 
 	/* Y2K compliant :) */
@@ -757,6 +843,62 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 		dmonth = G_DATE_BAD_MONTH;
 	}
 
+	t->tm_sec = ss;
+	t->tm_min = mm;
+	t->tm_hour = hh;
+	t->tm_mday = day;
+	t->tm_mon = dmonth - 1;
+	t->tm_year = year - 1900;
+	t->tm_wday = 0;
+	t->tm_yday = 0;
+	t->tm_isdst = -1;
+
+	mktime(t);
+
+	return TRUE;
+}
+
+time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
+{
+	static gchar monthstr[] = "JanFebMarAprMayJunJulAugSepOctNovDec";
+	gchar weekday[11];
+	gint day;
+	gchar month[10];
+	gint year;
+	gint hh, mm, ss;
+	gchar zone[6];
+	GDateMonth dmonth = G_DATE_BAD_MONTH;
+	struct tm t;
+	gchar *p;
+	time_t timer;
+	time_t tz_offset;
+
+	if (procheader_scan_date_string(src, weekday, &day, month, &year,
+					&hh, &mm, &ss, zone) < 0) {
+		g_warning("Invalid date: %s\n", src);
+		if (dest && len > 0)
+			strncpy2(dest, src, len);
+		return 0;
+	}
+
+	/* Y2K compliant :) */
+	if (year < 1000) {
+		if (year < 50)
+			year += 2000;
+		else
+			year += 1900;
+	}
+
+	month[3] = '\0';
+	for (p = monthstr; *p != '\0'; p += 3) {
+		if (!strncasecmp(p, month, 3)) {
+			dmonth = (gint)(p - monthstr) / 3 + 1;
+			break;
+		}
+	}
+	if (*p == '\0')
+		g_warning("Invalid month: %s\n", month);
+
 	t.tm_sec = ss;
 	t.tm_min = mm;
 	t.tm_hour = hh;
@@ -768,7 +910,9 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	t.tm_isdst = -1;
 
 	timer = mktime(&t);
-	timer += tzoffset_sec(&timer) - remote_tzoffset_sec(zone);
+	tz_offset = remote_tzoffset_sec(zone);
+	if (tz_offset != -1)
+		timer += tzoffset_sec(&timer) - tz_offset;
 
 	if (dest)
 		procheader_date_get_localtime(dest, len, timer);
@@ -789,27 +933,34 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 		strftime(dest, len, default_format, lt);
 }
 
-gint get_header_from_msginfo(MsgInfo *msginfo, gchar *buf, gint len,gchar *header)
+/* Added by Mel Hadasht on 27 Aug 2001 */
+/* Get a header from msginfo */
+gint procheader_get_header_from_msginfo(MsgInfo *msginfo, gchar *buf, gint len, gchar *header)
 {
-       gchar *file;
-       FILE *fp;
-       HeaderEntry hentry[]={{header,NULL,TRUE},
-                                    {NULL,NULL,FALSE}};
-       gint val;
-       g_return_if_fail(msginfo != NULL);
-       file = procmsg_get_message_file_path(msginfo);
-       if ((fp = fopen(file, "r")) == NULL) {
+	gchar *file;
+	FILE *fp;
+	HeaderEntry hentry[]={ { header, NULL, TRUE  },
+                               { NULL,   NULL, FALSE } };
+	gint val;
+       
+	g_return_val_if_fail(msginfo != NULL, -1);
+	file = procmsg_get_message_file_path(msginfo);
+	if ((fp = fopen(file, "rb")) == NULL) {
                FILE_OP_ERROR(file, "fopen");
                g_free(file);
-               return;
-        }
-       val=procheader_get_one_field(buf,len, fp, hentry);
-     	if (fclose(fp) == EOF) {
+               return -1;
+	}
+	val = procheader_get_one_field(buf,len, fp, hentry);
+	if (fclose(fp) == EOF) {
 		FILE_OP_ERROR(file, "fclose");
 		unlink(file);
+		g_free(file);
 		return -1;
 	}
-       if (val == -1)
-          return -1;
-       return 0;
+
+	g_free(file);
+        if (val == -1)
+		return -1;
+
+	return 0;
 }
