@@ -44,8 +44,8 @@
 #include "prefs_common.h"
 #include "passphrase.h"
 #include "select-keys.h"
-#include "sigstatus.h"
 #include "rfc2015.h"
+#include "alertpanel.h"
 
 #define DIM(v)     (sizeof(v)/sizeof((v)[0]))
 
@@ -61,14 +61,6 @@ static char *mime_version_name[] = {
     NULL
 };
 
-
-struct passphrase_cb_info_s {
-    GpgmeCtx c;
-    int did_it;
-};
-
-static char *create_boundary (void);
-
 #if 0
 static void dump_mimeinfo (const char *text, MimeInfo *x)
 {
@@ -111,14 +103,6 @@ static void dump_part ( MimeInfo *mimeinfo, FILE *fp )
 }
 #endif
 
-void
-rfc2015_disable_all (void)
-{
-    /* FIXME: set a flag, so that we don't bother the user with failed
-     * gpgme messages */
-}
-
-
 void
 rfc2015_secure_remove (const char *fname)
 {
@@ -129,386 +113,7 @@ rfc2015_secure_remove (const char *fname)
 }
 
 
-static const gchar *
-sig_status_to_string (GpgmeSigStat status)
-{
-    const gchar *result;
 
-    switch (status) {
-      case GPGME_SIG_STAT_NONE:
-        result = _("Oops: Signature not verified");
-        break;
-      case GPGME_SIG_STAT_NOSIG:
-        result = _("No signature found");
-        break;
-      case GPGME_SIG_STAT_GOOD:
-        result = _("Good signature");
-        break;
-      case GPGME_SIG_STAT_BAD:
-        result = _("BAD signature");
-        break;
-      case GPGME_SIG_STAT_NOKEY:
-        result = _("No public key to verify the signature");
-        break;
-      case GPGME_SIG_STAT_ERROR:
-        result = _("Error verifying the signature");
-        break;
-      case GPGME_SIG_STAT_DIFF:
-        result = _("Different results for signatures");
-        break;
-      default:
-	result = _("Error: Unknown status");
-	break;
-    }
-
-    return result;
-}
-
-static const gchar *
-sig_status_with_name (GpgmeSigStat status)
-{
-    const gchar *result;
-
-    switch (status) {
-      case GPGME_SIG_STAT_NONE:
-        result = _("Oops: Signature not verified");
-        break;
-      case GPGME_SIG_STAT_NOSIG:
-        result = _("No signature found");
-        break;
-      case GPGME_SIG_STAT_GOOD:
-        result = _("Good signature from \"%s\"");
-        break;
-      case GPGME_SIG_STAT_BAD:
-        result = _("BAD signature  from \"%s\"");
-        break;
-      case GPGME_SIG_STAT_NOKEY:
-        result = _("No public key to verify the signature");
-        break;
-      case GPGME_SIG_STAT_ERROR:
-        result = _("Error verifying the signature");
-        break;
-      case GPGME_SIG_STAT_DIFF:
-        result = _("Different results for signatures");
-        break;
-      default:
-	result = _("Error: Unknown status");
-	break;
-    }
-
-    return result;
-}
-
-static void
-sig_status_for_key(GString *str, GpgmeCtx ctx, GpgmeSigStat status, 
-		   GpgmeKey key, const gchar *fpr)
-{
-	gint idx = 0;
-	const char *uid;
-
-	uid = gpgme_key_get_string_attr (key, GPGME_ATTR_USERID, NULL, idx);
-	if (uid == NULL) {
-		g_string_sprintfa (str, "%s\n",
-				   sig_status_to_string (status));
-		if ((fpr != NULL) && (*fpr != '\0'))
-			g_string_sprintfa (str, "Key fingerprint: %s\n", fpr);
-		g_string_append (str, _("Cannot find user ID for this key."));
-		return;
-	}
-	g_string_sprintfa (str, sig_status_with_name (status), uid);
-	g_string_append (str, "\n");
-
-	while (1) {
-		uid = gpgme_key_get_string_attr (key, GPGME_ATTR_USERID,
-						 NULL, ++idx);
-		if (uid == NULL)
-			break;
-		g_string_sprintfa (str, _("                aka \"%s\"\n"),
-				   uid);
-	}
-}
-
-static gchar *
-sig_status_full (GpgmeCtx ctx)
-{
-	GString *str;
-	gint sig_idx = 0;
-	GpgmeError err;
-	GpgmeSigStat status;
-	GpgmeKey key;
-	const char *fpr;
-	time_t created;
-	struct tm *ctime_val;
-	char ctime_str[80];
-	gchar *retval;
-
-	str = g_string_new ("");
-
-	fpr = gpgme_get_sig_status (ctx, sig_idx, &status, &created);
-	while (fpr != NULL) {
-		if (created != 0) {
-			ctime_val = localtime (&created);
-			strftime (ctime_str, sizeof (ctime_str), "%c", 
-				  ctime_val);
-			g_string_sprintfa (str,
-					   _("Signature made %s\n"),
-					   ctime_str);
-		}
-		err = gpgme_get_sig_key (ctx, sig_idx, &key);
-		if (err != 0) {
-			g_string_sprintfa (str, "%s\n",
-					   sig_status_to_string (status));
-			if ((fpr != NULL) && (*fpr != '\0'))
-				g_string_sprintfa (str, 
-						   _("Key fingerprint: %s\n"),
-						   fpr);
-		} else {
-			sig_status_for_key (str, ctx, status, key, fpr);
-			gpgme_key_unref (key);
-		}
-		g_string_append (str, "\n\n");
-
-		fpr = gpgme_get_sig_status (ctx, ++sig_idx, &status, &created);
-	}
-
-	retval = str->str;
-	g_string_free (str, FALSE);
-	return retval;
-}
-
-static void check_signature (MimeInfo *mimeinfo, MimeInfo *partinfo, FILE *fp)
-{
-    GpgmeCtx ctx = NULL;
-    GpgmeError err;
-    GpgmeData sig = NULL, text = NULL;
-    GpgmeSigStat status = GPGME_SIG_STAT_NONE;
-    GpgmegtkSigStatus statuswindow = NULL;
-    const char *result = NULL;
-
-    if (prefs_common.gpg_signature_popup)
-	statuswindow = gpgmegtk_sig_status_create ();
-
-    err = gpgme_new (&ctx);
-    if (err) {
-	debug_print ("gpgme_new failed: %s\n", gpgme_strerror (err));
-	goto leave;
-    }
-
-    /* don't include the last character (LF). It does not belong to the
-     * signed text */
-    err = gpgme_data_new_from_filepart (&text, NULL, fp,
-					mimeinfo->children->fpos,
-					mimeinfo->children->size ?
-					(mimeinfo->children->size - 1) : 0 );
-    if (!err)
-	err = gpgme_data_new_from_filepart (&sig, NULL, fp,
-					    partinfo->fpos, partinfo->size);
-    if (err) {
-        debug_print ("gpgme_data_new_from_filepart failed: %s\n",
-		   gpgme_strerror (err));
-        goto leave;
-    }
-
-    err = gpgme_op_verify (ctx, sig, text, &status);
-    if (err) 
-        debug_print ("gpgme_op_verify failed: %s\n", gpgme_strerror (err));
-
-    /* FIXME: check what the heck this sig_status_full stuff is.
-     * it should better go into sigstatus.c */
-    g_free (partinfo->sigstatus_full);
-    partinfo->sigstatus_full = sig_status_full (ctx);
-
-leave:
-    result = gpgmegtk_sig_status_to_string(status);
-    debug_print("verification status: %s\n", result);
-    if (prefs_common.gpg_signature_popup)
-	gpgmegtk_sig_status_update (statuswindow,ctx);
-
-    g_assert (!err); /* FIXME: Hey: this may indeed happen */
-    g_free (partinfo->sigstatus);
-    partinfo->sigstatus = g_strdup (result);
-
-    gpgme_data_release (sig);
-    gpgme_data_release (text);
-    gpgme_release (ctx);
-    if (prefs_common.gpg_signature_popup)
-	gpgmegtk_sig_status_destroy (statuswindow);
-}
-
-static const char *
-passphrase_cb (void *opaque, const char *desc, void **r_hd)
-{
-    struct passphrase_cb_info_s *info = opaque;
-    GpgmeCtx ctx = info ? info->c : NULL;
-    const char *pass;
-
-    if (!desc) {
-        /* FIXME: cleanup by looking at **r_hd */
-        return NULL;
-    }
-
-    gpgmegtk_set_passphrase_grab (prefs_common.passphrase_grab);
-    debug_print ("%% requesting passphrase for `%s': ", desc );
-    pass = gpgmegtk_passphrase_mbox (desc);
-    if (!pass) {
-        debug_print ("%% cancel passphrase entry");
-        gpgme_cancel (ctx);
-    }
-    else
-        debug_print ("%% sending passphrase");
-
-    return pass;
-}
-
-/*
- * Copy a gpgme data object to a temporary file and
- * return this filename 
- */
-#if 0
-static char *
-copy_gpgmedata_to_temp (GpgmeData data, guint *length)
-{
-    static int id;
-    char *tmp;
-    FILE *fp;
-    char buf[100];
-    size_t nread;
-    GpgmeError err;
-    
-    tmp = g_strdup_printf("%s%cgpgtmp.%08x",
-                          get_mime_tmp_dir(), G_DIR_SEPARATOR, ++id );
-
-    if ((fp = fopen(tmp, "w")) == NULL) {
-        FILE_OP_ERROR(tmp, "fopen");
-        g_free(tmp);
-        return NULL;
-    }
-
-    err = gpgme_data_rewind ( data );
-    if (err)
-        debug_print ("gpgme_data_rewind failed: %s\n", gpgme_strerror (err));
-
-    while (!(err = gpgme_data_read (data, buf, 100, &nread))) {
-        fwrite ( buf, nread, 1, fp );
-    }
-
-    if (err != GPGME_EOF)
-        debug_print ("gpgme_data_read failed: %s\n", gpgme_strerror (err));
-
-    fclose (fp);
-    *length = nread;
-
-    return tmp;
-}
-#endif
-
-static GpgmeData
-pgp_decrypt (MimeInfo *partinfo, FILE *fp)
-{
-    GpgmeCtx ctx = NULL;
-    GpgmeError err;
-    GpgmeData cipher = NULL, plain = NULL;
-    struct passphrase_cb_info_s info;
-
-    memset (&info, 0, sizeof info);
-
-    err = gpgme_new (&ctx);
-    if (err) {
-        debug_print ("gpgme_new failed: %s\n", gpgme_strerror (err));
-        goto leave;
-    }
-
-    err = gpgme_data_new_from_filepart (&cipher, NULL, fp,
-					partinfo->fpos, partinfo->size);
-    if (err) {
-        debug_print ("gpgme_data_new_from_filepart failed: %s\n",
-                     gpgme_strerror (err));
-        goto leave;
-    }
-
-    err = gpgme_data_new (&plain);
-    if (err) {
-        debug_print ("gpgme_new failed: %s\n", gpgme_strerror (err));
-        goto leave;
-    }
-
-    if (!getenv("GPG_AGENT_INFO")) {
-        info.c = ctx;
-        gpgme_set_passphrase_cb (ctx, passphrase_cb, &info);
-    } 
-
-    err = gpgme_op_decrypt (ctx, cipher, plain);
-
-leave:
-    gpgme_data_release (cipher);
-    if (err) {
-        debug_print ("decryption failed: %s\n", gpgme_strerror (err));
-        gpgme_data_release (plain);
-        plain = NULL;
-    }
-    else
-        debug_print ("** decryption succeeded");
-
-    gpgme_release (ctx);
-    return plain;
-}
-
-MimeInfo * rfc2015_find_signature (MimeInfo *mimeinfo)
-{
-    MimeInfo *partinfo;
-    int n = 0;
-
-    if (!mimeinfo)
-        return NULL;
-    if (g_strcasecmp (mimeinfo->content_type, "multipart/signed"))
-        return NULL;
-
-    debug_print ("** multipart/signed encountered");
-
-    /* check that we have at least 2 parts of the correct type */
-    for (partinfo = mimeinfo->children;
-         partinfo != NULL; partinfo = partinfo->next) {
-        if (++n > 1  && !g_strcasecmp (partinfo->content_type,
-				       "application/pgp-signature"))
-            break;
-    }
-
-    return partinfo;
-}
-
-gboolean rfc2015_has_signature (MimeInfo *mimeinfo)
-{
-    return rfc2015_find_signature (mimeinfo) != NULL;
-}
-
-void rfc2015_check_signature (MimeInfo *mimeinfo, FILE *fp)
-{
-    MimeInfo *partinfo;
-
-    partinfo = rfc2015_find_signature (mimeinfo);
-    if (!partinfo)
-        return;
-
-#if 0
-    g_message ("** yep, it is a pgp signature");
-    dump_mimeinfo ("gpg-signature", partinfo );
-    dump_part (partinfo, fp );
-    dump_mimeinfo ("signed text", mimeinfo->children );
-    dump_part (mimeinfo->children, fp);
-#endif
-
-    check_signature (mimeinfo, partinfo, fp);
-}
-
-int rfc2015_is_encrypted (MimeInfo *mimeinfo)
-{
-    if (!mimeinfo)
-        return 0;
-    if (g_strcasecmp (mimeinfo->content_type, "multipart/encrypted"))
-        return 0;
-    /* fixme: we should schek the protocol parameter */
-    return 1;
-}
 
 static int
 name_cmp(const char *a, const char *b)
@@ -550,112 +155,6 @@ headerp(char *p, char **names)
     return names[i] != NULL;
 }
 
-
-#define DECRYPTION_ABORT() \
-{ \
-    procmime_mimeinfo_free_all(tmpinfo); \
-    msginfo->decryption_failed = 1; \
-    return; \
-}
-
-void rfc2015_decrypt_message (MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
-{
-    static int id;
-    MimeInfo *tmpinfo, *partinfo;
-    int ver_ok = 0;
-    char *fname;
-    GpgmeData plain;
-    FILE *dstfp;
-    size_t nread;
-    char buf[BUFFSIZE];
-    GpgmeError err;
-
-    g_return_if_fail (msginfo != NULL);
-    g_return_if_fail (mimeinfo != NULL);
-    g_return_if_fail (fp != NULL);
-    g_return_if_fail (mimeinfo->mime_type == MIME_MULTIPART);
-
-    debug_print ("** decrypting multipart/encrypted message");
-
-    /* skip headers */
-    if (fseek(fp, mimeinfo->fpos, SEEK_SET) < 0)
-        perror("fseek");
-    tmpinfo = procmime_scan_mime_header(fp);
-    if (!tmpinfo || tmpinfo->mime_type != MIME_MULTIPART) {
-        DECRYPTION_ABORT();
-    }
-
-    procmime_scan_multipart_message(tmpinfo, fp);
-
-    /* check that we have the 2 parts */
-    partinfo = tmpinfo->children;
-    if (!partinfo || !partinfo->next) {
-        DECRYPTION_ABORT();
-    }
-    if (!g_strcasecmp (partinfo->content_type, "application/pgp-encrypted")) {
-        /* Fixme: check that the version is 1 */
-        ver_ok = 1;
-    }
-    partinfo = partinfo->next;
-    if (ver_ok &&
-        !g_strcasecmp (partinfo->content_type, "application/octet-stream")) {
-        if (partinfo->next)
-            g_warning ("oops: pgp_encrypted with more than 2 parts");
-    }
-    else {
-        DECRYPTION_ABORT();
-    }
-
-    debug_print ("** yep, it is pgp encrypted");
-
-    plain = pgp_decrypt (partinfo, fp);
-    if (!plain) {
-        DECRYPTION_ABORT();
-    }
-
-    fname = g_strdup_printf("%s%cplaintext.%08x",
-			    get_mime_tmp_dir(), G_DIR_SEPARATOR, ++id);
-
-    if ((dstfp = fopen(fname, "w")) == NULL) {
-        FILE_OP_ERROR(fname, "fopen");
-        g_free(fname);
-        DECRYPTION_ABORT();
-    }
-
-    /* write the orginal header to the new file */
-    if (fseek(fp, tmpinfo->fpos, SEEK_SET) < 0)
-        perror("fseek");
-
-    while (fgets(buf, sizeof(buf), fp)) {
-        if (headerp (buf, content_names))
-            continue;
-        if (buf[0] == '\r' || buf[0] == '\n')
-            break;
-        fputs (buf, dstfp);
-    }
-
-    err = gpgme_data_rewind (plain);
-    if (err)
-        debug_print ("gpgme_data_rewind failed: %s\n", gpgme_strerror (err));
-
-    while (!(err = gpgme_data_read (plain, buf, sizeof(buf), &nread))) {
-        fwrite (buf, nread, 1, dstfp);
-    }
-
-    if (err != GPGME_EOF) {
-        debug_print ("gpgme_data_read failed: %s\n", gpgme_strerror (err));
-    }
-
-    fclose (dstfp);
-    procmime_mimeinfo_free_all(tmpinfo);
-
-    msginfo->plaintext_file = fname;
-    msginfo->decryption_failed = 0;
-}
-
-#undef DECRYPTION_ABORT
-
-
 /*
  * plain contains an entire mime object.
  * Encrypt it and return an GpgmeData object with the encrypted version of
@@ -682,7 +181,7 @@ pgp_encrypt ( GpgmeData plain, GpgmeRecipients rset )
         cipher = NULL;
     }
     else {
-        debug_print ("** encryption succeeded");
+        debug_print ("** encryption succeeded\n");
     }
 
     gpgme_release (ctx);
@@ -747,7 +246,9 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
     GpgmeRecipients rset = NULL;
     size_t nread;
     int mime_version_seen = 0;
-    char *boundary = create_boundary ();
+    char *boundary;
+
+    boundary = generate_mime_boundary ("Encrypt");
 
     /* Create the list of recipients */
     rset = gpgmegtk_recipient_selection (recp_list);
@@ -771,7 +272,7 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
     }
 
     /* get the content header lines from the source */
-    clineidx=0;
+    clineidx = 0;
     saved_last = 0;
     while (!err && fgets(buf, sizeof(buf), fp)) {
         /* fixme: check for overlong lines */
@@ -785,13 +286,13 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
             continue;
         }
         if (saved_last) {
-            saved_last = 0;
             if (*buf == ' ' || *buf == '\t') {
-                char *last = clines[clineidx-1];
-                clines[clineidx-1] = g_strconcat (last, buf, NULL);
+                char *last = clines[clineidx - 1];
+                clines[clineidx - 1] = g_strconcat (last, buf, NULL);
                 g_free (last);
                 continue;
             }
+            saved_last = 0;
         }
 
         if (headerp (buf, mime_version_name)) 
@@ -875,7 +376,7 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
 
     if (ascii_armored) {
         fprintf(fp, 
-            "Content-Type: text/plain; charset=us-ascii\r\n"
+            "Content-Type: text/plain; charset=US-ASCII\r\n"
             "Content-Transfer-Encoding: 7bit\r\n"  
             "\r\n");
     } else {
@@ -914,10 +415,9 @@ rfc2015_encrypt (const char *file, GSList *recp_list, gboolean ascii_armored)
     /* and the final boundary */
     if (!ascii_armored) {
         fprintf (fp,
-	        "\r\n"
-	        "--%s--\r\n"
-	        "\r\n",
-            boundary);
+		 "\r\n"
+		 "--%s--\r\n",
+	         boundary);
     }
     fflush (fp);
     if (ferror (fp)) {
@@ -942,9 +442,11 @@ failure:
 /* 
  * plain contains an entire mime object.  Sign it and return an
  * GpgmeData object with the signature of it or NULL in case of error.
+ * r_siginfo returns an XML object with information about the signature.
  */
 static GpgmeData
-pgp_sign (GpgmeData plain, GSList *key_list)
+pgp_sign (GpgmeData plain, GSList *key_list, gboolean clearsign,
+	  char **r_siginfo)
 {
     GSList *p;
     GpgmeCtx ctx = NULL;
@@ -952,6 +454,7 @@ pgp_sign (GpgmeData plain, GSList *key_list)
     GpgmeData sig = NULL;
     struct passphrase_cb_info_s info;
 
+    *r_siginfo = NULL;
     memset (&info, 0, sizeof info);
 
     err = gpgme_new (&ctx);
@@ -963,7 +466,7 @@ pgp_sign (GpgmeData plain, GSList *key_list)
 
     if (!getenv("GPG_AGENT_INFO")) {
         info.c = ctx;
-        gpgme_set_passphrase_cb (ctx, passphrase_cb, &info);
+        gpgme_set_passphrase_cb (ctx, gpgmegtk_passphrase_cb, &info);
     }
     gpgme_set_textmode (ctx, 1);
     gpgme_set_armor (ctx, 1);
@@ -979,10 +482,15 @@ pgp_sign (GpgmeData plain, GSList *key_list)
 
     if (err)
 	goto leave;
-    err = gpgme_op_sign (ctx, plain, sig, GPGME_SIG_MODE_DETACH);
+    err = gpgme_op_sign
+	(ctx, plain, sig,
+	 clearsign ? GPGME_SIG_MODE_CLEAR : GPGME_SIG_MODE_DETACH);
+    if (!err)
+        *r_siginfo = gpgme_get_op_info (ctx, 0);
 
 leave:
     if (err) {
+        gpgmegtk_free_passphrase();
         debug_print ("signing failed: %s\n", gpgme_strerror (err));
         gpgme_data_release (sig);
         sig = NULL;
@@ -995,6 +503,67 @@ leave:
     return sig;
 }
 
+/*
+ * Find TAG in XML and return a pointer into xml set just behind the
+ * closing angle.  Return NULL if not found. 
+ */
+static const char *
+find_xml_tag (const char *xml, const char *tag)
+{
+    int taglen = strlen (tag);
+    const char *s = xml;
+ 
+    while ( (s = strchr (s, '<')) ) {
+        s++;
+        if (!strncmp (s, tag, taglen)) {
+            const char *s2 = s + taglen;
+            if (*s2 == '>' || isspace (*(const unsigned char*)s2) ) {
+                /* found */
+                while (*s2 && *s2 != '>') /* skip attributes */
+                    s2++;
+                /* fixme: do need to handle angles inside attribute vallues? */
+                return *s2? (s2+1):NULL;
+            }
+        }
+        while (*s && *s != '>') /* skip to end of tag */
+            s++;
+    }
+    return NULL;
+}
+
+
+/*
+ * Extract the micalg from an GnupgOperationInfo XML container.
+ */
+static char *
+extract_micalg (char *xml)
+{
+    const char *s;
+
+    s = find_xml_tag (xml, "GnupgOperationInfo");
+    if (s) {
+        const char *s_end = find_xml_tag (s, "/GnupgOperationInfo");
+        s = find_xml_tag (s, "signature");
+        if (s && s_end && s < s_end) {
+            const char *s_end2 = find_xml_tag (s, "/signature");
+            if (s_end2 && s_end2 < s_end) {
+                s = find_xml_tag (s, "micalg");
+                if (s && s < s_end2) {
+                    s_end = strchr (s, '<');
+                    if (s_end) {
+                        char *p = g_malloc (s_end - s + 1);
+                        memcpy (p, s, s_end - s);
+                        p[s_end-s] = 0;
+                        return p;
+                    }
+                }
+            }
+        }
+    }
+    return NULL;
+}
+
+
 /*
  * Sign the file and replace its content with the signed one.
  */
@@ -1011,7 +580,11 @@ rfc2015_sign (const char *file, GSList *key_list)
     GpgmeData sigdata = NULL;
     size_t nread;
     int mime_version_seen = 0;
-    char *boundary = create_boundary ();
+    char *boundary;
+    char *micalg = NULL;
+    char *siginfo;
+
+    boundary = generate_mime_boundary ("Signature");
 
     /* Open the source file */
     if ((fp = fopen(file, "rb")) == NULL) {
@@ -1042,13 +615,13 @@ rfc2015_sign (const char *file, GSList *key_list)
             continue;
         }
         if (saved_last) {
-            saved_last = 0;
             if (*buf == ' ' || *buf == '\t') {
                 char *last = clines[clineidx - 1];
                 clines[clineidx - 1] = g_strconcat (last, buf, NULL);
                 g_free (last);
                 continue;
             }
+            saved_last = 0;
         }
 
         if (headerp (buf, mime_version_name)) 
@@ -1081,7 +654,11 @@ rfc2015_sign (const char *file, GSList *key_list)
         goto failure;
     }
 
-    sigdata = pgp_sign (plain, key_list);
+    sigdata = pgp_sign (plain, key_list, FALSE, &siginfo); 
+    if (siginfo) {
+	micalg = extract_micalg (siginfo);
+	free (siginfo);
+    }
     if (!sigdata) 
         goto failure;
 
@@ -1117,13 +694,17 @@ rfc2015_sign (const char *file, GSList *key_list)
     header = NULL;
 
     if (!mime_version_seen) 
-        fputs ("MIME-Version: 1\r\n", fp);
+        fputs ("MIME-Version: 1.0\r\n", fp);
     fprintf (fp, "Content-Type: multipart/signed; "
-		 "protocol=\"application/pgp-signature\";\r\n"
-		 " boundary=\"%s\"\r\n", boundary );
+             "protocol=\"application/pgp-signature\";\r\n");
+    if (micalg)
+        fprintf (fp, " micalg=\"%s\";\r\n", micalg);
+    fprintf (fp, " boundary=\"%s\"\r\n", boundary);
 
     /* Part 1: signed material */
-    fprintf (fp, "\r\n--%s\r\n", boundary);
+    fprintf (fp, "\r\n"
+                 "--%s\r\n",
+                 boundary);
     err = gpgme_data_rewind (plain);
     if (err) {
         debug_print ("gpgme_data_rewind on plain failed: %s\n",
@@ -1139,7 +720,9 @@ rfc2015_sign (const char *file, GSList *key_list)
     }
 
     /* Part 2: signature */
-    fprintf (fp, "\r\n--%s\r\n", boundary);
+    fprintf (fp, "\r\n"
+                 "--%s\r\n",
+                 boundary);
     fputs ("Content-Type: application/pgp-signature\r\n"
 	   "\r\n", fp);
 
@@ -1159,7 +742,9 @@ rfc2015_sign (const char *file, GSList *key_list)
     }
 
     /* Final boundary */
-    fprintf (fp, "\r\n--%s--\r\n\r\n", boundary);
+    fprintf (fp, "\r\n"
+                 "--%s--\r\n",
+                 boundary);
     fflush (fp);
     if (ferror (fp)) {
         FILE_OP_ERROR (file, "fwrite");
@@ -1170,6 +755,7 @@ rfc2015_sign (const char *file, GSList *key_list)
     gpgme_data_release (plain);
     gpgme_data_release (sigdata);
     g_free (boundary);
+    g_free (micalg);
     return 0;
 
 failure:
@@ -1179,62 +765,95 @@ failure:
     gpgme_data_release (plain);
     gpgme_data_release (sigdata);
     g_free (boundary);
+    g_free (micalg);
     return -1; /* error */
 }
 
 
-/****************
- * Create a new boundary in a way that it is very unlikely that this
- * will occur in the following text.  It would be easy to ensure
- * uniqueness if everything is either quoted-printable or base64
- * encoded (note that conversion is allowed), but because MIME bodies
- * may be nested, it may happen that the same boundary has already
- * been used. We avoid scanning the message for conflicts and hope the
- * best.
- *
- *   boundary := 0*69<bchars> bcharsnospace
- *   bchars := bcharsnospace / " "
- *   bcharsnospace := DIGIT / ALPHA / "'" / "(" / ")" /
- *                    "+" / "_" / "," / "-" / "." /
- *                    "/" / ":" / "=" / "?"  
+/*
+ * Sign the file with clear text and replace its content with the signed one.
  */
-
-static char *
-create_boundary (void)
+gint
+rfc2015_clearsign (const gchar *file, GSList *key_list)
 {
-    static char tbl[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-			"abcdefghijklmnopqrstuvwxyz"
-			"1234567890'()+_,./:=?";
-    char buf[17];
-    int i, equal;
-    int pid;
+    FILE *fp;
+    gchar buf[BUFFSIZE];
+    GpgmeError err;
+    GpgmeData text = NULL;
+    GpgmeData sigdata = NULL;
+    size_t nread;
+    gchar *siginfo;
 
-    pid = getpid();
-
-    /* We make the boundary depend on the pid, so that all running
-     * processed generate different values even when they have been
-     * started within the same second and srand48(time(NULL)) has been
-     * used.  I can't see whether this is really an advantage but it
-     * doesn't do any harm.
-     */
-    equal = -1;
-    for(i = 0; i < sizeof(buf) - 1; i++) {
-	buf[i] = tbl[(lrand48() ^ pid) % (sizeof(tbl) - 1)]; /* fill with random */
-	if(buf[i] == '=' && equal == -1)
-	    equal = i;
-    }
-    buf[i] = 0;
-
-    /* now make sure that we do have the sequence "=." in it which cannot
-     * be matched by quoted-printable or base64 encoding */
-    if(equal != -1 && (equal+1) < i)
-	buf[equal+1] = '.';
-    else {
-	buf[0] = '=';
-	buf[1] = '.';
+    if ((fp = fopen(file, "rb")) == NULL) {
+	FILE_OP_ERROR(file, "fopen");
+	goto failure;
     }
 
-    return g_strdup(buf);
+    err = gpgme_data_new(&text);
+    if (err) {
+	debug_print("gpgme_data_new failed: %s\n", gpgme_strerror(err));
+	goto failure;
+    }
+
+    while (!err && fgets(buf, sizeof(buf), fp)) {
+	err = gpgme_data_write(text, buf, strlen(buf));
+    }
+    if (ferror(fp)) {
+	FILE_OP_ERROR(file, "fgets");
+	goto failure;
+    }
+    if (err) {
+	debug_print("gpgme_data_write failed: %s\n", gpgme_strerror(err));
+	goto failure;
+    }
+
+    sigdata = pgp_sign(text, key_list, TRUE, &siginfo);
+    if (siginfo) {
+	g_free(siginfo);
+    }
+    if (!sigdata)
+	goto failure;
+
+    if (fclose(fp) == EOF) {
+	FILE_OP_ERROR(file, "fclose");
+	fp = NULL;
+	goto failure;
+    }
+    if ((fp = fopen(file, "wb")) == NULL) {
+	FILE_OP_ERROR(file, "fopen");
+	goto failure;
+    }
+
+    err = gpgme_data_rewind(sigdata);
+    if (err) {
+	debug_print("gpgme_data_rewind on sigdata failed: %s\n",
+		    gpgme_strerror(err));
+	goto failure;
+    }
+
+    while (!(err = gpgme_data_read(sigdata, buf, sizeof(buf), &nread))) {
+	fwrite(buf, nread, 1, fp);
+    }
+    if (err != GPGME_EOF) {
+	debug_print("gpgme_data_read failed: %s\n", gpgme_strerror(err));
+	goto failure;
+    }
+
+    if (fclose(fp) == EOF) {
+	FILE_OP_ERROR(file, "fclose");
+	fp = NULL;
+	goto failure;
+    }
+    gpgme_data_release(text);
+    gpgme_data_release(sigdata);
+    return 0;
+
+failure:
+    if (fp)
+	fclose(fp);
+    gpgme_data_release(text);
+    gpgme_data_release(sigdata);
+    return -1;
 }
 
 #endif /* USE_GPGME */
